@@ -2,19 +2,19 @@
 /*
  * common.h
  * Copyright (C) Kevin DeKorte 2006 <kdekorte@gmail.com>
- * 
+ *
  * common.h is free software.
- * 
+ *
  * You may redistribute it and/or modify it under the terms of the
  * GNU General Public License, as published by the Free Software
  * Foundation; either version 2 of the License, or (at your option)
  * any later version.
- * 
+ *
  * common.h is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with common.h.  If not, write to:
  * 	The Free Software Foundation, Inc.,
@@ -34,6 +34,7 @@
 
 #define MIXER			"mixer"
 #define CACHE_SIZE		"cache_size"
+#define PLUGIN_CACHE_SIZE		"plugin_cache_size"
 #define OSDLEVEL		"osdlevel"
 #define PPLEVEL		"pplevel"
 #define SOFTVOL			"softvol"
@@ -59,10 +60,14 @@
 #define SUBTITLECOLOR	"subtitlecolor"
 #define SUBTITLEOUTLINE "subtitleoutline"
 #define SUBTITLESHADOW "subtitleshadow"
+#define SUBTITLE_MARGIN "subtitle_margin"
 #define TRACKER_POSITION "tracker_position"
 
 #define VOLUME	"volume"
+#define AUDIO_CHANNELS "audio_channels"
+#define USE_HW_AUDIO "use_hw_audio"
 #define USE_MEDIAKEYS		"use_mediakeys"
+#define USE_DEFAULTPL		"use_defaultpl"
 #define DISABLE_COVER_ART_FETCH "disable_cover_art_fetch"
 #define FULLSCREEN	"fullscreen"
 
@@ -83,13 +88,16 @@
 #define WINDOW_HEIGHT	"window_height"
 #define WINDOW_WIDTH	"window_width"
 
+#define RESIZE_ON_NEW_MEDIA "resize_on_new_media"
 #define KEEP_ON_TOP		"keep_on_top"
 #define SINGLE_INSTANCE "single_instance"
 #define REPLACE_AND_PLAY "replace_and_play"
+#define BRING_TO_FRONT  "bring_to_front"
 #define SHOW_NOTIFICATION "show_notification"
 #define SHOW_STATUS_ICON "show_status_icon"
 #define SHOW_SUBTITLES "show_subtitles"
 
+
 // JavaScript Playstates
 #define STATE_UNDEFINED     0
 #define STATE_STOPPED       1
@@ -139,12 +147,15 @@ typedef struct _IdleData {
     gint streaming;
     gchar progress_text[1024];
     gdouble volume;
+    gint mplayer_volume;
     gboolean mute;
     gchar vol_tooltip[128];
     gint x;
     gint y;
     gint last_x;
     gint last_y;
+    gint original_h;
+    gint original_w;
     gint width;
     gint height;
     gboolean videopresent;
@@ -197,6 +208,9 @@ typedef struct _ThreadData {
     gint streaming;
     gint player_window;
     gint playlist;
+    gint restart_second;
+    gint start_second;
+    gint play_length;
     gboolean done;
 } ThreadData;
 
@@ -216,6 +230,11 @@ enum {
     DEMUXER_COLUMN,
     VIDEO_WIDTH_COLUMN,
     VIDEO_HEIGHT_COLUMN,
+    PLAY_ORDER_COLUMN,
+    ADD_ORDER_COLUMN,
+    START_COLUMN,
+    END_COLUMN,
+    PLAYABLE_COLUMN,
     N_COLUMNS
 };
 
@@ -232,6 +251,7 @@ typedef struct _MetaData {
     gchar *demuxer;
     gint width;
     gint height;
+    gboolean playable;
 } MetaData;
 
 typedef struct _LangMenu {
@@ -260,6 +280,7 @@ gint max_buckets[METER_BARS];
 
 gchar *lastfile;
 gint cache_size;
+gint plugin_cache_size;
 gboolean forcecache;
 gint osdlevel;
 gint pplevel;
@@ -282,16 +303,21 @@ gint play_x, play_y;
 gint last_x, last_y;
 gint last_window_width, last_window_height;
 gint stored_window_width, stored_window_height;
+gboolean adjusting;
 gchar vm[10];
 gchar *vo;
 gchar *ao;
 gchar *mixer;
+gint audio_channels;
+gboolean use_hw_audio;
 gboolean disable_deinterlace;
 gboolean disable_framedrop;
 gboolean disable_context_menu;
 gboolean disable_fullscreen;
 gboolean disable_pause_on_click;
 gint loop;
+gint start_second;
+gint play_length;
 gint random_order;
 gboolean dontplaynext;
 gboolean autopause;
@@ -323,17 +349,21 @@ gint auto_hide_timeout;
 gboolean always_hide_after_timeout;
 gchar *subtitlefont;
 gdouble subtitle_scale;
+gdouble subtitle_delay;
 gchar *subtitle_codepage;
 gchar *subtitle_color;
 gboolean subtitle_outline;
 gboolean subtitle_shadow;
+gint subtitle_margin;
 gboolean quit_on_complete;
 gchar *mplayer_bin;
 gchar *extraopts;
+gboolean resize_on_new_media;
 gboolean use_pulse_flat_volume;
 gboolean single_instance;
 gboolean new_instance;
 gboolean replace_and_play;
+gboolean bring_to_front;
 gboolean use_pausing_keep_force;
 gboolean show_notification;
 gboolean show_status_icon;
@@ -341,11 +371,11 @@ gboolean load_tracks_from_gpod;
 gchar *gpod_mount_point;
 gboolean disable_cover_art_fetch;
 gboolean updating_recent;
-gint thumb_position;
 gboolean large_buttons;
 gint button_size;
 
 gboolean remember_loc;
+gboolean use_remember_loc;
 gint loc_window_x;
 gint loc_window_y;
 gint loc_window_height;
@@ -365,7 +395,10 @@ GThread *thread;
 GMutex *slide_away;
 GCond *mplayer_complete_cond;
 
+GThreadPool *retrieve_metadata_pool;
+
 gboolean use_mediakeys;
+gboolean use_defaultpl;
 gboolean dvdnav_title_is_menu;
 
 gboolean qt_disabled;
@@ -379,9 +412,14 @@ GArray *data;
 GArray *max_data;
 gboolean reading_af_export;
 
+gboolean sub_source_file;
+
+// layout variables
+gint non_fs_width;
+gint non_fs_height;
+
 // playlist stuff
 GtkListStore *playliststore;
-GtkListStore *nonrandomplayliststore;
 GtkTreeIter iter;
 GtkTreeSelection *selection;
 GtkWidget *list;
@@ -392,11 +430,12 @@ GmPrefStore *gmp_store;
 
 GtkWidget *create_window(gint windowid);
 void show_window(gint windowid);
+void present_main_window();
 gint get_player_window();
-void adjust_paned_rules();
-void reset_paned_rules();
+void adjust_layout();
 gboolean hide_buttons(void *data);
 gboolean show_copyurl(void *data);
+
 gboolean set_media_info(void *data);
 gboolean set_media_label(void *data);
 gboolean set_cover_art(gpointer pixbuf);
@@ -415,6 +454,7 @@ gboolean set_rew(void *data);
 gboolean set_prev(void *data);
 gboolean set_next(void *data);
 gboolean set_quit(void *data);
+gboolean set_kill_mplayer(void *data);
 gboolean set_position(void *data);
 gboolean set_volume(void *data);
 gboolean set_fullscreen(void *data);
@@ -424,6 +464,8 @@ gboolean set_window_visible(void *data);
 gboolean set_update_gui(void *data);
 gboolean set_subtitle_visibility(void *data);
 gboolean set_item_add_info(void *data);
+gboolean set_metadata(gpointer data);
+
 void remove_langs(GtkWidget * item, gpointer data);
 gboolean set_new_lang_menu(gpointer data);
 gboolean set_new_audio_menu(gpointer data);
@@ -441,6 +483,7 @@ void dbus_send_event(gchar * event, gint button);
 void dbus_unhook();
 void dbus_enable_screensaver();
 void dbus_disable_screensaver();
+void menuitem_edit_random_callback(GtkMenuItem * menuitem, void *data);
 
 gboolean update_audio_meter(gpointer data);
 
@@ -450,6 +493,11 @@ gpointer launch_player(gpointer data);
 gboolean update_mplayer_config();
 gboolean read_mplayer_config();
 
+void retrieve_metadata(gpointer data, gpointer user_data);
+
+gchar *default_playlist;
+gboolean safe_to_save_default_playlist;
+
 #ifdef GTK2_12_ENABLED
 GtkRecentManager *recent_manager;
 void recent_manager_changed_callback(GtkRecentManager * recent_manager, gpointer data);
