@@ -66,10 +66,8 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     gchar *path4;
     GString *xml;
     gchar *xml_string;
-    GtkTreePath *treepath;
     gint source_id;
     gint bitrate;
-    gchar *buf = NULL;
 
     message_type = dbus_message_get_type(message);
     sender = dbus_message_get_sender(message);
@@ -100,34 +98,13 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
             // printf("Path matched %s\n", dbus_message_get_path(message));
             if (message_type == DBUS_MESSAGE_TYPE_SIGNAL) {
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Open") == 0) {
-                    mplayer_shutdown();
+                    g_idle_add(set_kill_mplayer,NULL);
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
                         if (strlen(s) > 0) {
                             if (strcmp(rpconsole, "NONE") == 0 || control_instance == FALSE) {
-                                gtk_list_store_clear(playliststore);
-                                gtk_list_store_clear(nonrandomplayliststore);
-                                selection = NULL;
-                                if (!uri_exists(s) && !streaming_media(s)) {
-                                    buf = g_filename_to_uri(s, NULL, NULL);
-                                } else {
-                                    buf = g_strdup(s);
-                                }
-                                playlist = detect_playlist(buf);
-                                if (!playlist) {
-                                    add_item_to_playlist(buf, playlist);
-                                } else {
-                                    if (!parse_playlist(buf)) {
-                                        add_item_to_playlist(buf, playlist);
-                                    }
-                                }
-                                g_free(buf);
-                                if (gtk_tree_model_get_iter_first
-                                    (GTK_TREE_MODEL(playliststore), &iter)) {
-                                    play_iter(&iter);
-                                }
-
+                                g_idle_add(clear_playlist_and_play, g_strdup(s));
                             }
                         }
 
@@ -138,36 +115,12 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "OpenPlaylist") == 0) {
-                    mplayer_shutdown();
+                    g_idle_add(set_kill_mplayer,NULL);
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
-                        gtk_list_store_clear(playliststore);
-                        gtk_list_store_clear(nonrandomplayliststore);
                         selection = NULL;
-                        if (!uri_exists(s) && !streaming_media(s)) {
-                            buf = g_filename_to_uri(s, NULL, NULL);
-                        } else {
-                            buf = g_strdup(s);
-                        }
-                        if (!parse_playlist(buf)) {
-                            add_item_to_playlist(buf, 1);
-                        }
-                        g_free(buf);
-                        if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                            play_iter(&iter);
-                        }
-
-                        if (GTK_IS_TREE_SELECTION(selection)) {
-                            treepath =
-                                gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
-                            gtk_tree_selection_select_path(selection, treepath);
-                            if (GTK_IS_WIDGET(list))
-                                gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), treepath, NULL,
-                                                             FALSE, 0, 0);
-                            gtk_tree_path_free(treepath);
-                        }
-                        g_idle_add(set_update_gui, NULL);
+                        g_idle_add(clear_playlist_and_play, g_strdup(s));
 
                     } else {
                         dbus_error_free(&error);
@@ -176,7 +129,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "OpenButton") == 0) {
-                    mplayer_shutdown();
+                    g_idle_add(set_kill_mplayer,NULL);
                     dbus_error_init(&error);
                     if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &s,
                                               DBUS_TYPE_STRING, &hrefid, DBUS_TYPE_INVALID)) {
@@ -192,22 +145,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
                         if (strlen(s) > 0) {
-                            if (!uri_exists(s) && !streaming_media(s)) {
-                                buf = g_filename_to_uri(s, NULL, NULL);
-                            } else {
-                                buf = g_strdup(s);
-                            }
-                            playlist = detect_playlist(buf);
-                            if (!playlist) {
-                                add_item_to_playlist(buf, playlist);
-                            } else {
-                                if (!parse_playlist(buf)) {
-                                    add_item_to_playlist(buf, playlist);
-                                }
-                            }
-                            g_free(buf);
-                            g_idle_add(set_update_gui, NULL);
-                            // if play on add is set, play the last item on the playlist
+                            g_idle_add(add_to_playlist_and_play, g_strdup(s));
                         }
                     } else {
                         dbus_error_free(&error);
@@ -230,12 +168,12 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Close") == 0) {
-                    mplayer_shutdown();
+                    g_idle_add(set_kill_mplayer,NULL);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Quit") == 0) {
-                    mplayer_shutdown();
+                    g_idle_add(set_kill_mplayer,NULL);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
@@ -367,6 +305,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
 					dbus_unhook();
                     gtk_main_quit();
 */
+                    dontplaynext = TRUE;
                     g_idle_add(set_quit, idledata);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
@@ -469,7 +408,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                         g_idle_add(set_progress_value, idledata);
                         if (idledata->cachepercent > 0.99 && idledata->retry_on_full_cache) {
                             if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                                play_iter(&iter);
+                                play_iter(&iter, 0);
                             }
                         }
                     } else {
@@ -624,19 +563,19 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_STRING, &keyname,
                          DBUS_TYPE_INVALID)) {
-                        if (g_strncasecmp(keyname, "Play", strlen("Play")) == 0) {
+                        if (g_ascii_strncasecmp(keyname, "Play", strlen("Play")) == 0) {
                             idledata->fromdbus = TRUE;
                             g_idle_add(set_play, idledata);
                         }
-                        if (g_strncasecmp(keyname, "Stop", strlen("Stop")) == 0) {
+                        if (g_ascii_strncasecmp(keyname, "Stop", strlen("Stop")) == 0) {
                             idledata->fromdbus = TRUE;
                             g_idle_add(set_stop, idledata);
                         }
-                        if (g_strncasecmp(keyname, "Previous", strlen("Previous")) == 0) {
+                        if (g_ascii_strncasecmp(keyname, "Previous", strlen("Previous")) == 0) {
                             idledata->fromdbus = TRUE;
                             g_idle_add(set_prev, idledata);
                         }
-                        if (g_strncasecmp(keyname, "Next", strlen("Next")) == 0) {
+                        if (g_ascii_strncasecmp(keyname, "Next", strlen("Next")) == 0) {
                             idledata->fromdbus = TRUE;
                             g_idle_add(set_next, idledata);
                         }
@@ -788,7 +727,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
                 if (dbus_message_is_method_call(message, "com.gnome.mplayer", "GetCacheSize")) {
                     reply_message = dbus_message_new_method_return(message);
-                    dbus_message_append_args(reply_message, DBUS_TYPE_INT32, &cache_size,
+                    dbus_message_append_args(reply_message, DBUS_TYPE_INT32, &plugin_cache_size,
                                              DBUS_TYPE_INVALID);
                     dbus_connection_send(connection, reply_message, NULL);
                     dbus_message_unref(reply_message);
@@ -1156,23 +1095,15 @@ gboolean dbus_hookup(gint windowid, gint controlid)
             exit(1);
         }
     }
-
+    screensaver_disabled = FALSE;
 
     return TRUE;
 }
 
 void dbus_unhook()
 {
-
-    DBusMessage *message;
-
     if (connection != NULL) {
-        message =
-            dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
-                                         "org.gnome.ScreenSaver", "UnInhibit");
-        dbus_message_append_args(message, DBUS_TYPE_INT32, &cookie, DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, message, NULL);
-        dbus_message_unref(message);
+        dbus_enable_screensaver();
         // dbus_connection_close(connection);
         dbus_connection_unref(connection);
         connection = NULL;
@@ -1181,27 +1112,45 @@ void dbus_unhook()
 
 void dbus_enable_screensaver()
 {
+#if SM_INHIBIT || SS_INHIBIT
     DBusMessage *message;
+#endif
 
-    if (connection != NULL) {
+    if (connection != NULL && screensaver_disabled) {
+#ifdef SM_INHIBIT
+        message =
+            dbus_message_new_method_call("org.gnome.SessionManager", "/org/gnome/SessionManager",
+                                         "org.gnome.SessionManager", "UnInhibit");
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &sm_cookie, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+#endif
+#ifdef SS_INHIBIT
         message =
             dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
                                          "org.gnome.ScreenSaver", "UnInhibit");
-        dbus_message_append_args(message, DBUS_TYPE_INT32, &cookie, DBUS_TYPE_INVALID);
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &ss_cookie, DBUS_TYPE_INVALID);
         dbus_connection_send(connection, message, NULL);
         dbus_message_unref(message);
+#endif
+        screensaver_disabled = FALSE;
     }
 }
 
 void dbus_disable_screensaver()
 {
+#if SM_INHIBIT || SS_INHIBIT
     DBusError error;
     DBusMessage *reply_message;
     DBusMessage *message;
     const gchar *app;
     const gchar *reason;
+    gint flags;
+#endif
 
     if (connection != NULL) {
+
+#ifdef SS_INHIBIT
         dbus_error_init(&error);
         message =
             dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
@@ -1211,12 +1160,37 @@ void dbus_disable_screensaver()
         dbus_message_append_args(message, DBUS_TYPE_STRING, &app, DBUS_TYPE_STRING, &reason,
                                  DBUS_TYPE_INVALID);
         reply_message = dbus_connection_send_with_reply_and_block(connection, message, 200, &error);
+
         if (reply_message) {
-            dbus_message_get_args(reply_message, &error, DBUS_TYPE_INT32, &cookie, NULL);
+            dbus_message_get_args(reply_message, &error, DBUS_TYPE_INT32, &ss_cookie, NULL);
             dbus_message_unref(reply_message);
         }
 
         dbus_message_unref(message);
         dbus_error_free(&error);
+#endif
+
+#ifdef SM_INHIBIT
+        dbus_error_init(&error);
+        message =
+            dbus_message_new_method_call("org.gnome.SessionManager", "/org/gnome/SessionManager",
+                                         "org.gnome.SessionManager", "Inhibit");
+        app = g_strdup_printf("gnome-mplayer");
+        reason = g_strdup_printf("playback");
+        flags = 8;
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &app, DBUS_TYPE_UINT32,
+                                 &(idledata->windowid), DBUS_TYPE_STRING, &reason, DBUS_TYPE_UINT32,
+                                 &flags, DBUS_TYPE_INVALID);
+        reply_message = dbus_connection_send_with_reply_and_block(connection, message, 200, &error);
+
+        if (reply_message) {
+            dbus_message_get_args(reply_message, &error, DBUS_TYPE_INT32, &sm_cookie, NULL);
+            dbus_message_unref(reply_message);
+        }
+
+        dbus_message_unref(message);
+        dbus_error_free(&error);
+#endif
+        screensaver_disabled = TRUE;
     }
 }
