@@ -30,5 +30,7 @@
 #include "support.h"
 
 DBusConnection *connection;
-uint cookie;
+uint ss_cookie;
+uint sm_cookie;
+gboolean screensaver_disabled;
 gboolean dbus_hookup(gint windowid, gint controlid);
