@@ -2,19 +2,19 @@
 /*
  * gui.h
  * Copyright (C) Kevin DeKorte 2006 <kdekorte@gmail.com>
- * 
+ *
  * gui.h is free software.
- * 
+ *
  * You may redistribute it and/or modify it under the terms of the
  * GNU General Public License, as published by the Free Software
  * Foundation; either version 2 of the License, or (at your option)
  * any later version.
- * 
+ *
  * gui.h is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with gui.h.  If not, write to:
  * 	The Free Software Foundation, Inc.,
@@ -23,6 +23,9 @@
  */
 //#include <X11/Xlib.h>
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -110,6 +113,8 @@ GtkMenuItem *menuitem_view_sep2;
 GtkMenuItem *menuitem_view_subtitles;
 GtkMenuItem *menuitem_view_smaller_subtitle;
 GtkMenuItem *menuitem_view_larger_subtitle;
+GtkMenuItem *menuitem_view_decrease_subtitle_delay;
+GtkMenuItem *menuitem_view_increase_subtitle_delay;
 GtkMenuItem *menuitem_view_sep5;
 GtkMenuItem *menuitem_view_angle;
 GtkMenuItem *menuitem_view_controls;
@@ -147,10 +152,20 @@ GtkWidget *fixed;
 GtkWidget *media_hbox;
 GtkWidget *media_label;
 GtkWidget *cover_art;
+GtkWidget *audio_meter;
 GtkWidget *details_vbox;
 GtkWidget *details_table;
-GtkWidget *audio_meter;
 
+GtkWidget *details_video_size;
+GtkWidget *details_video_format;
+GtkWidget *details_video_codec;
+GtkWidget *details_video_fps;
+GtkWidget *details_video_bitrate;
+GtkWidget *details_video_chapters;
+GtkWidget *details_audio_codec;
+GtkWidget *details_audio_channels;
+GtkWidget *details_audio_bitrate;
+GtkWidget *details_audio_samplerate;
 
 GtkWidget *drawing_area;
 GdkPixbuf *pb_play;
@@ -163,8 +178,10 @@ GdkPixbuf *pb_next;
 GdkPixbuf *pb_prev;
 GdkPixbuf *pb_menu;
 GdkPixbuf *pb_icon;
-GdkPixbuf *pb_logo;
 GdkPixbuf *pb_button;
+
+GList *icon_list;
+
 GtkWidget *button_event_box;
 GtkWidget *image_button;
 
@@ -202,7 +219,6 @@ GtkWidget *image_fs;
 GtkWidget *image_icon;
 GtkTooltips *tooltip;
 GtkTooltips *volume_tip;
-GtkTooltips *progress_tip;
 
 #ifdef GTK2_12_ENABLED
 GtkStatusIcon *status_icon;
@@ -212,8 +228,11 @@ GtkWidget *config_show_status_icon;
 GtkWidget *config_vo;
 GtkWidget *config_ao;
 GtkWidget *config_mixer;
+GtkWidget *config_audio_channels;
+GtkWidget *config_use_hw_audio;
 GtkWidget *config_volume;
 GtkWidget *config_cachesize;
+GtkWidget *config_plugin_cache_size;
 GtkWidget *config_osdlevel;
 GtkWidget *config_deinterlace;
 GtkWidget *config_framedrop;
@@ -224,12 +243,16 @@ GtkWidget *config_details_visible;
 GtkWidget *config_vertical_layout;
 GtkWidget *config_single_instance;
 GtkWidget *config_replace_and_play;
+GtkWidget *config_bring_to_front;
+GtkWidget *config_resize_on_new_media;
 GtkWidget *config_pause_on_click;
 GtkWidget *config_softvol;
 GtkWidget *config_forcecache;
 GtkWidget *config_verbose;
 GtkWidget *config_show_notification;
 GtkWidget *config_thumb_position;
+GtkWidget *config_use_mediakeys;
+GtkWidget *config_use_defaultpl;
 
 GtkWidget *config_alang;
 GtkWidget *config_slang;
@@ -243,6 +266,7 @@ GtkWidget *config_subtitle_codepage;
 GtkWidget *config_subtitle_color;
 GtkWidget *config_subtitle_outline;
 GtkWidget *config_subtitle_shadow;
+GtkWidget *config_subtitle_margin;
 GtkWidget *config_show_subtitles;
 
 GtkWidget *config_qt;
@@ -278,11 +302,11 @@ GtkWidget *adv_saturation;
 GtkAccelGroup *accel_group;
 
 glong last_movement_time;
-gboolean mouse_down_in_progress;
 
 gboolean popup_handler(GtkWidget * widget, GdkEvent * event, void *data);
 gboolean delete_callback(GtkWidget * widget, GdkEvent * event, void *data);
 void config_close(GtkWidget * widget, void *data);
+gboolean allocate_fixed_callback(GtkWidget * widget, GtkAllocation * allocation, gpointer data);
 
 gboolean rew_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
 gboolean play_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
@@ -304,7 +328,10 @@ void menuitem_fs_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_showcontrols_callback(GtkCheckMenuItem * menuitem, void *data);
 void menuitem_quit_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_details_callback(GtkMenuItem * menuitem, void *data);
+void menuitem_view_decrease_subtitle_delay_callback(GtkMenuItem * menuitem, void *data);
+void menuitem_view_increase_subtitle_delay_callback(GtkMenuItem * menuitem, void *data);
 void clear_playlist(GtkWidget * widget, void *data);
+void create_playlist_widget();
 gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
                                 gint x, gint y, GtkSelectionData * selection_data,
                                 guint info, guint t, gpointer data);
@@ -312,6 +339,7 @@ void create_folder_progress_window();
 void destroy_folder_progress_window();
 void update_status_icon();
 void setup_accelerators();
+void update_details_table();
 
 gint get_height();
 gint get_width();
