@@ -2,19 +2,19 @@
 /*
  * main.c
  * Copyright (C) Kevin DeKorte 2006 <kdekorte@gmail.com>
- * 
+ *
  * main.c is free software.
- * 
+ *
  * You may redistribute it and/or modify it under the terms of the
  * GNU General Public License, as published by the Free Software
  * Foundation; either version 2 of the License, or (at your option)
  * any later version.
- * 
+ *
  * main.c is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with main.c.  If not, write to:
  * 	The Free Software Foundation, Inc.,
@@ -78,6 +78,11 @@ static GOptionEntry entries[] = {
      NULL},
     {"cache", 0, 0, G_OPTION_ARG_INT, &cache_size, N_("Set cache size"),
      NULL},
+    {"ss", 0, 0, G_OPTION_ARG_INT, &start_second, N_("Start Second (default to 0)"),
+     NULL},
+    {"endpos", 0, 0, G_OPTION_ARG_INT, &play_length,
+     N_("Length of media to play from start second"),
+     NULL},
     {"forcecache", 0, 0, G_OPTION_ARG_NONE, &forcecache, N_("Force cache usage on streaming sites"),
      NULL},
     {"disabledeinterlace", 0, 0, G_OPTION_ARG_NONE, &disable_deinterlace,
@@ -138,7 +143,54 @@ static GOptionEntry entries[] = {
     {NULL}
 };
 
-gint play_iter(GtkTreeIter * playiter)
+gboolean play(void *data);
+
+void play_next()
+{
+    gchar *filename;
+    gint count;
+    PlayData *p = NULL;
+
+    if (next_item_in_playlist(&iter)) {
+        if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+            gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
+                               COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
+            g_strlcpy(idledata->info, filename, 4096);
+            g_idle_add(set_media_info, idledata);
+            p = (PlayData *) g_malloc(sizeof(PlayData));
+            g_strlcpy(p->uri, filename, 4096);
+            p->playlist = playlist;
+            g_idle_add(play, p);
+            g_free(filename);
+        }
+    } else {
+        // printf("end of thread playlist is empty\n");
+        if (loop) {
+            if (first_item_in_playlist(&iter)) {
+                gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,
+                                   &filename, COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
+                g_strlcpy(idledata->info, filename, 4096);
+                g_idle_add(set_media_info, idledata);
+                p = (PlayData *) g_malloc(sizeof(PlayData));
+                g_strlcpy(p->uri, filename, 4096);
+                p->playlist = playlist;
+                g_idle_add(play, p);
+                g_free(filename);
+            }
+        } else {
+            idledata->fullscreen = 0;
+            g_idle_add(set_fullscreen, idledata);
+            g_idle_add(set_stop, idledata);
+        }
+
+        if (quit_on_complete) {
+            g_idle_add(set_quit, idledata);
+        }
+    }
+}
+
+
+gint play_iter(GtkTreeIter * playiter, gint restart_second)
 {
 
     ThreadData *thread_data = (ThreadData *) g_new0(ThreadData, 1);
@@ -156,8 +208,10 @@ gint play_iter(GtkTreeIter * playiter)
     gchar *audio_codec;
     gchar *video_codec = NULL;
     gchar *demuxer = NULL;
+    gboolean playable = TRUE;
     gint width;
     gint height;
+    gfloat length_value;
     gint i;
     gpointer pixbuf;
     gchar *buffer = NULL;
@@ -183,7 +237,8 @@ gint play_iter(GtkTreeIter * playiter)
                            DEMUXER_COLUMN, &demuxer,
                            COVERART_COLUMN, &pixbuf,
                            SUBTITLE_COLUMN, &subtitle,
-                           COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
+                           COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist,
+                           PLAYABLE_COLUMN, &playable, -1);
         if (GTK_IS_TREE_SELECTION(selection)) {
             path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), playiter);
             if (path) {
@@ -198,7 +253,8 @@ gint play_iter(GtkTreeIter * playiter)
         }
         gtk_list_store_set(playliststore, playiter, COUNT_COLUMN, count + 1, -1);
     } else {
-        printf("iter is invalid, nothing to play\n");
+        if (verbose > 1)
+            printf("iter is invalid, nothing to play\n");
         return 0;
     }
 
@@ -212,14 +268,65 @@ gint play_iter(GtkTreeIter * playiter)
     while (state != QUIT) {
         gtk_main_iteration();
     }
-    reset_paned_rules();
 
+    while (gtk_events_pending())
+        gtk_main_iteration();
+
+    // wait for metadata to be available on this item
+    if (!streaming_media(uri) && !device_name(uri)) {
+        i = 0;
+        if (playable) {
+            while (demuxer == NULL && i < 10) {
+                g_free(title);
+                g_free(artist);
+                g_free(album);
+                g_free(audio_codec);
+                g_free(video_codec);
+                g_free(demuxer);
+                g_free(subtitle);
+                if (gtk_list_store_iter_is_valid(playliststore, playiter)) {
+                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), playiter, LENGTH_VALUE_COLUMN,
+                                       &length_value, DESCRIPTION_COLUMN, &title, ARTIST_COLUMN,
+                                       &artist, ALBUM_COLUMN, &album, AUDIO_CODEC_COLUMN,
+                                       &audio_codec, VIDEO_CODEC_COLUMN, &video_codec,
+                                       VIDEO_WIDTH_COLUMN, &width, VIDEO_HEIGHT_COLUMN, &height,
+                                       DEMUXER_COLUMN, &demuxer, COVERART_COLUMN, &pixbuf,
+                                       SUBTITLE_COLUMN, &subtitle, COUNT_COLUMN, &count,
+                                       PLAYLIST_COLUMN, &playlist, PLAYABLE_COLUMN, &playable, -1);
+                    if (!playable) {
+                        if (verbose)
+                            printf("%s is not marked as playable (%i)\n", uri, i);
+                        play_next();
+                        return 0;
+                    }
+                } else {
+                    if (verbose)
+                        printf("Current iter is not valid\n");
+                    return 1;   // error condition
+                }
+                gtk_main_iteration();
+                i++;
+                if (demuxer == NULL)
+                    g_usleep(10000);
+            }
+        } else {
+            if (verbose)
+                printf("%s is not marked as playable\n", uri);
+            play_next();
+            return 0;
+        }
+
+    }
     // reset audio meter
     for (i = 0; i < METER_BARS; i++) {
         buckets[i] = 0;
         max_buckets[i] = 0;
     }
 
+    gmtk_media_tracker_set_text(tracker, _("Playing"));
+    gmtk_media_tracker_set_position(tracker, (gfloat) restart_second);
+    gmtk_media_tracker_set_length(tracker, length_value);
+
     message = g_strdup_printf("<small>\n");
     if (title == NULL) {
         title = g_filename_display_basename(uri);
@@ -315,32 +422,34 @@ gint play_iter(GtkTreeIter * playiter)
         g_strlcpy(thread_data->subtitle, subtitle, 1024);
         g_free(subtitle);
     }
-#ifdef HAVE_ASOUNDLIB
-    if (!softvol && ao != NULL
-        && (g_ascii_strcasecmp(ao, "alsa") == 0
-            || (use_pulse_flat_volume && g_ascii_strcasecmp(ao, "pulse") == 0))) {
-        volume = (gint) get_alsa_volume();
-        idledata->volume = volume;
-#if GTK2_12_ENABLED
-        gtk_scale_button_set_value(GTK_SCALE_BUTTON(vol_slider), volume);
-#else
-        gtk_range_set_value(GTK_RANGE(vol_slider), volume);
-#endif
-    }
-#endif
-#if GTK2_12_ENABLED
-    volume = gtk_scale_button_get_value(GTK_SCALE_BUTTON(vol_slider));
-#else
-    volume = gtk_range_get_value(GTK_RANGE(vol_slider));
-#endif
+    /*
+       #ifdef HAVE_ASOUNDLIB
+       if (!softvol && ao != NULL
+       && (g_ascii_strcasecmp(ao, "alsa") == 0
+       || (use_pulse_flat_volume && g_ascii_strcasecmp(ao, "pulse") == 0))) {
+       volume = (gint) get_alsa_volume(TRUE);
+       idledata->volume = volume;
+       #if GTK2_12_ENABLED
+       gtk_scale_button_set_value(GTK_SCALE_BUTTON(vol_slider), volume);
+       #else
+       gtk_range_set_value(GTK_RANGE(vol_slider), volume);
+       #endif
+       }
+       #endif
 
+       #if GTK2_12_ENABLED
+       volume = gtk_scale_button_get_value(GTK_SCALE_BUTTON(vol_slider));
+       #else
+       volume = gtk_range_get_value(GTK_RANGE(vol_slider));
+       #endif
+     */
     if (g_ascii_strcasecmp(thread_data->filename, "") != 0) {
         if (!device_name(thread_data->filename) && !streaming_media(thread_data->filename)) {
             if (!g_file_test(thread_data->filename, G_FILE_TEST_EXISTS)) {
                 error_msg = g_strdup_printf("%s not found\n", thread_data->filename);
                 dialog =
                     gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error_msg);
+                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", error_msg);
                 gtk_window_set_title(GTK_WINDOW(dialog), "GNOME MPlayer Error");
                 gtk_dialog_run(GTK_DIALOG(dialog));
                 gtk_widget_destroy(dialog);
@@ -351,7 +460,7 @@ gint play_iter(GtkTreeIter * playiter)
 #ifdef GTK2_12_ENABLED
 #ifdef GIO_ENABLED
     // don't put it on the recent list, if it is running in plugin mode
-    if (control_id == 0 && !streaming_media (uri)) {
+    if (control_id == 0 && !streaming_media(uri)) {
         recent_data = (GtkRecentData *) g_new0(GtkRecentData, 1);
         if (artist != NULL && strlen(artist) > 0) {
             recent_data->display_name = g_strdup_printf("%s - %s", artist, title);
@@ -375,10 +484,10 @@ gint play_iter(GtkTreeIter * playiter)
         g_object_unref(file);
         recent_data->app_name = g_strdup("gnome-mplayer");
         recent_data->app_exec = g_strdup("gnome-mplayer %u");
-		if (recent_data->mime_type != NULL) {
-	        gtk_recent_manager_add_full(recent_manager, uri, recent_data);
-	        g_free(recent_data->mime_type);
-		}
+        if (recent_data->mime_type != NULL) {
+            gtk_recent_manager_add_full(recent_manager, uri, recent_data);
+            g_free(recent_data->mime_type);
+        }
         g_free(recent_data->app_name);
         g_free(recent_data->app_exec);
         g_free(recent_data);
@@ -419,21 +528,39 @@ gint play_iter(GtkTreeIter * playiter)
     last_y = 0;
     idledata->width = width;
     idledata->height = height;
+    idledata->original_h = -1;
+    idledata->original_w = -1;
     if (width > 0 && height > 0) {
         idledata->videopresent = 1;
     } else {
         idledata->videopresent = 0;
     }
+
+    if (idledata->audio_codec != NULL && strlen(idledata->audio_codec) > 0) {
+        idledata->audiopresent = 1;
+    } else {
+        idledata->audiopresent = 0;
+    }
+
     idledata->x = 0;
     idledata->y = 0;
+    idledata->cachepercent = -1.0;
     g_strlcpy(idledata->info, uri, 1024);
     set_media_info(idledata);
 
     streaming = 0;
+    subtitle_delay = 0.0;
+
+    gm_store = gm_pref_store_new("gnome-mplayer");
+    forcecache = gm_pref_store_get_boolean(gm_store, FORCECACHE);
+    gm_pref_store_free(gm_store);
 
     if (thread_data->filename != NULL && strlen(thread_data->filename) != 0) {
         thread_data->player_window = 0;
         thread_data->playlist = playlist;
+        thread_data->restart_second = restart_second;
+        thread_data->start_second = start_second;
+        thread_data->play_length = play_length;
         thread_data->streaming = streaming_media(thread_data->uri);
         idledata->streaming = thread_data->streaming;
         streaming = thread_data->streaming;
@@ -451,6 +578,8 @@ gint play_iter(GtkTreeIter * playiter)
             idledata->videopresent = 1;
             gtk_widget_set_size_request(drawing_area, 16, 16);
             gtk_widget_show_all(fixed);
+        } else {
+            g_idle_add(resize_window, idledata);
         }
 
         if (g_ascii_strcasecmp(uri, "dvdnav://") == 0) {
@@ -464,11 +593,16 @@ gint play_iter(GtkTreeIter * playiter)
         if (autostart) {
             g_idle_add(hide_buttons, idledata);
             js_state = STATE_PLAYING;
+            while (gtk_events_pending())
+                gtk_main_iteration();
             thread = g_thread_create(launch_player, thread_data, TRUE, NULL);
+            if (thread == NULL)
+                printf("Unable to launch player thread\n");
         }
         autostart = 1;
     }
 
+
     return 0;
 }
 
@@ -489,6 +623,7 @@ int main(int argc, char *argv[])
     GOptionContext *context;
     gint i;
     struct sigaction sa;
+    gboolean playiter = FALSE;
 
 #ifdef GIO_ENABLED
     GFile *file;
@@ -579,12 +714,15 @@ int main(int argc, char *argv[])
     disable_animation = FALSE;
     auto_hide_timeout = 3;
     use_mediakeys = TRUE;
+    use_defaultpl = FALSE;
     mplayer_bin = NULL;
     single_instance = FALSE;
     disable_deinterlace = TRUE;
     details_visible = FALSE;
     replace_and_play = FALSE;
+    bring_to_front = FALSE;
     keep_on_top = FALSE;
+    resize_on_new_media = FALSE;
     use_pausing_keep_force = FALSE;
     show_notification = TRUE;
     show_status_icon = TRUE;
@@ -599,13 +737,17 @@ int main(int argc, char *argv[])
     vo = NULL;
     use_pulse_flat_volume = FALSE;
     dvdnav_title_is_menu = FALSE;
-    thumb_position = 3;
     data = NULL;
     max_data = NULL;
     details_table = NULL;
     large_buttons = FALSE;
     button_size = 16;
     lastguistate = -1;
+    non_fs_height = 0;
+    non_fs_width = 0;
+    use_hw_audio = FALSE;
+    start_second = 0;
+    play_length = 0;
 
     sa.sa_handler = hup_handler;
     sigemptyset(&sa.sa_mask);
@@ -634,6 +776,10 @@ int main(int argc, char *argv[])
     g_free(uri);
     uri = NULL;
 
+    default_playlist =
+        g_strdup_printf("file://%s/gnome-mplayer/default.pls", g_get_user_config_dir());
+    safe_to_save_default_playlist = TRUE;
+
     gm_store = gm_pref_store_new("gnome-mplayer");
     gmp_store = gm_pref_store_new("gecko-mediaplayer");
     mixer = gm_pref_store_get_string(gm_store, MIXER);
@@ -642,6 +788,8 @@ int main(int argc, char *argv[])
 #ifndef HAVE_ASOUNDLIB
     volume = gm_pref_store_get_int(gm_store, VOLUME);
 #endif
+    audio_channels = gm_pref_store_get_int(gm_store, AUDIO_CHANNELS);
+    use_hw_audio = gm_pref_store_get_boolean(gm_store, USE_HW_AUDIO);
     fullscreen = gm_pref_store_get_boolean(gm_store, FULLSCREEN);
     softvol = gm_pref_store_get_boolean(gm_store, SOFTVOL);
     forcecache = gm_pref_store_get_boolean(gm_store, FORCECACHE);
@@ -661,7 +809,8 @@ int main(int argc, char *argv[])
     auto_hide_timeout =
         gm_pref_store_get_int_with_default(gm_store, AUTOHIDETIMEOUT, auto_hide_timeout);
     disable_cover_art_fetch = gm_pref_store_get_boolean(gm_store, DISABLE_COVER_ART_FETCH);
-    use_mediakeys = gm_pref_store_get_boolean(gm_store, USE_MEDIAKEYS);
+    use_mediakeys = gm_pref_store_get_boolean_with_default(gm_store, USE_MEDIAKEYS, use_mediakeys);
+    use_defaultpl = gm_pref_store_get_boolean_with_default(gm_store, USE_DEFAULTPL, use_defaultpl);
     metadata_codepage = gm_pref_store_get_string(gm_store, METADATACODEPAGE);
     subtitlefont = gm_pref_store_get_string(gm_store, SUBTITLEFONT);
     subtitle_scale = gm_pref_store_get_float(gm_store, SUBTITLESCALE);
@@ -672,6 +821,7 @@ int main(int argc, char *argv[])
     subtitle_color = gm_pref_store_get_string(gm_store, SUBTITLECOLOR);
     subtitle_outline = gm_pref_store_get_boolean(gm_store, SUBTITLEOUTLINE);
     subtitle_shadow = gm_pref_store_get_boolean(gm_store, SUBTITLESHADOW);
+    subtitle_margin = gm_pref_store_get_int(gm_store, SUBTITLE_MARGIN);
     showsubtitles = gm_pref_store_get_boolean_with_default(gm_store, SHOW_SUBTITLES, TRUE);
 
     qt_disabled = gm_pref_store_get_boolean(gmp_store, DISABLE_QT);
@@ -680,9 +830,13 @@ int main(int argc, char *argv[])
     dvx_disabled = gm_pref_store_get_boolean(gmp_store, DISABLE_DVX);
     midi_disabled = gm_pref_store_get_boolean(gmp_store, DISABLE_MIDI);
     embedding_disabled = gm_pref_store_get_boolean(gmp_store, DISABLE_EMBEDDING);
-    single_instance = gm_pref_store_get_boolean(gm_store, SINGLE_INSTANCE);
-    if (single_instance)
-        replace_and_play = gm_pref_store_get_boolean(gm_store, REPLACE_AND_PLAY);
+    if (embed_window == 0) {
+        single_instance = gm_pref_store_get_boolean(gm_store, SINGLE_INSTANCE);
+        if (single_instance) {
+            replace_and_play = gm_pref_store_get_boolean(gm_store, REPLACE_AND_PLAY);
+            bring_to_front = gm_pref_store_get_boolean(gm_store, BRING_TO_FRONT);
+        }
+    }
 
     mplayer_bin = gm_pref_store_get_string(gm_store, MPLAYER_BIN);
     if (mplayer_bin != NULL && !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
@@ -691,7 +845,6 @@ int main(int argc, char *argv[])
     }
     extraopts = gm_pref_store_get_string(gm_store, EXTRAOPTS);
     use_pulse_flat_volume = gm_pref_store_get_boolean(gm_store, USE_PULSE_FLAT_VOLUME);
-    thumb_position = gm_pref_store_get_int_with_default(gm_store, TRACKER_POSITION, thumb_position);
 
     remember_loc = gm_pref_store_get_boolean(gm_store, REMEMBER_LOC);
     loc_window_x = gm_pref_store_get_int(gm_store, WINDOW_X);
@@ -699,6 +852,7 @@ int main(int argc, char *argv[])
     loc_window_height = gm_pref_store_get_int(gm_store, WINDOW_HEIGHT);
     loc_window_width = gm_pref_store_get_int(gm_store, WINDOW_WIDTH);
     keep_on_top = gm_pref_store_get_boolean(gm_store, KEEP_ON_TOP);
+    resize_on_new_media = gm_pref_store_get_boolean(gm_store, RESIZE_ON_NEW_MEDIA);
     read_mplayer_config();
 
     context = g_option_context_new(_("[FILES...] - GNOME Media player based on MPlayer"));
@@ -727,6 +881,14 @@ int main(int argc, char *argv[])
         cache_size = gm_pref_store_get_int(gm_store, CACHE_SIZE);
     if (cache_size == 0)
         cache_size = 2000;
+
+    plugin_cache_size = gm_pref_store_get_int(gm_store, PLUGIN_CACHE_SIZE);
+    if (plugin_cache_size == 0)
+        plugin_cache_size = 2000;
+
+    if (control_id != 0)
+        cache_size = plugin_cache_size;
+
     gm_pref_store_free(gm_store);
     gm_pref_store_free(gmp_store);
 
@@ -740,18 +902,30 @@ int main(int argc, char *argv[])
         printf("Running with GIO support\n");
     }
 #endif
+#ifdef ENABLE_PANSCAN
+    if (verbose) {
+        printf("Running with panscan enabled (mplayer svn r29565 or higher required)\n");
+    }
+#endif
+
+    if (ao != NULL && g_ascii_strncasecmp(ao, "pulse", strlen("pulse")) == 0) {
+        // do nothing
+    } else {
+        // we have alsa or pulse flat volume now
+        use_pulse_flat_volume = TRUE;
+    }
 
     if (volume == -1) {
-        volume = (gint) get_alsa_volume();
-		if (!use_pulse_flat_volume) {
-		    if (ao != NULL && g_ascii_strncasecmp(ao, "pulse", strlen("pulse")) == 0) {
-		        if (verbose)
-		            printf
-		                ("Using pulse audio, setting volume to max (will be limited by mixer 100%% of %i%%)\n",
-		                 volume);
-		        volume = 100;
-		    }
-		}		
+        volume = (gint) get_alsa_volume(TRUE);
+        if (!use_pulse_flat_volume) {
+            if (ao != NULL && g_ascii_strncasecmp(ao, "pulse", strlen("pulse")) == 0) {
+                if (verbose)
+                    printf
+                        ("Using pulse audio in non-flat volume mode, setting volume to max (will be limited by mixer 100%% of %i%%)\n",
+                         volume);
+                volume = 100;
+            }
+        }
     } else {
         if (verbose)
             printf("Using volume of %i from gnome-mplayer preference\n", volume);
@@ -798,12 +972,8 @@ int main(int argc, char *argv[])
         gtk_list_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,
                            G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_STRING,
                            G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
-                           G_TYPE_INT);
-    nonrandomplayliststore =
-        gtk_list_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,
-                           G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_STRING,
-                           G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
-                           G_TYPE_INT);
+                           G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_FLOAT, G_TYPE_FLOAT,
+                           G_TYPE_BOOLEAN);
 
     create_window(embed_window);
 
@@ -821,6 +991,8 @@ int main(int argc, char *argv[])
     idledata->caching_complete = g_cond_new();
 #endif
 
+    retrieve_metadata_pool = g_thread_pool_new(retrieve_metadata, NULL, 10, TRUE, NULL);
+
     if (argv[fileindex] != NULL) {
         g_stat(argv[fileindex], &buf);
         if (verbose) {
@@ -856,7 +1028,7 @@ int main(int argc, char *argv[])
                 if (S_ISDIR(buf.st_mode)) {
                     add_item_to_playlist("dvd://", 0);
                     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
-                    play_iter(&iter);
+                    play_iter(&iter, 0);
                 } else {
                     uri = g_strdup_printf("file://%s", mnt->mnt_dir);
                     create_folder_progress_window();
@@ -868,7 +1040,8 @@ int main(int argc, char *argv[])
                         randomize_playlist(playliststore);
                     }
                     if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                        play_iter(&iter);
+                        // play_iter(&iter, 0);
+                        playiter = TRUE;
                     }
                 }
             } else {
@@ -879,7 +1052,8 @@ int main(int argc, char *argv[])
                 }
                 //play_file("cdda://", playlist);
                 if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                    play_iter(&iter);
+                    // play_iter(&iter, 0);
+                    playiter = TRUE;
                 }
             }
         } else if (S_ISDIR(buf.st_mode)) {
@@ -902,7 +1076,8 @@ int main(int argc, char *argv[])
                 randomize_playlist(playliststore);
             }
             if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                play_iter(&iter);
+                //play_iter(&iter, 0);
+                playiter = TRUE;
             }
 
         } else {
@@ -947,12 +1122,12 @@ int main(int argc, char *argv[])
                 randomize_playlist(playliststore);
             }
             if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                play_iter(&iter);
+                // play_iter(&iter, 0);
+                playiter = TRUE;
             }
         }
 
-    } 
-
+    }
 #ifdef HAVE_GPOD
     if (load_tracks_from_gpod) {
         gpod_mount_point = find_gpod_mount_point();
@@ -965,13 +1140,37 @@ int main(int argc, char *argv[])
     }
 #endif
 
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_details), details_visible);
 
     dbus_hookup(embed_window, control_id);
     show_window(embed_window);
-	if (argv[fileindex] == NULL && embed_window == 0) {
-        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_playlist), playlist_visible);
-    }	
+
+    if (playiter)
+        play_iter(&iter, 0);
+
+
+    if (argv[fileindex] == NULL && embed_window == 0) {
+        use_remember_loc = remember_loc;
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_playlist),
+                                       playlist_visible);
+    } else {
+        // prevents saving of a playlist with one item on it
+        use_defaultpl = FALSE;
+    }
+
+    if (single_instance && embed_window == 0) {
+        use_remember_loc = remember_loc;
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_playlist),
+                                       playlist_visible);
+    }
+
+    safe_to_save_default_playlist = FALSE;
+    if (use_defaultpl) {
+        create_folder_progress_window();
+        parse_playlist(default_playlist);
+        destroy_folder_progress_window();
+    }
+    safe_to_save_default_playlist = TRUE;
+
     gtk_main();
 
     return 0;
