@@ -2,19 +2,19 @@
 /*
  * playlist.c
  * Copyright (C) Kevin DeKorte 2006 <kdekorte@gmail.com>
- * 
+ *
  * playlist.c is free software.
- * 
+ *
  * You may redistribute it and/or modify it under the terms of the
  * GNU General Public License, as published by the Free Software
  * Foundation; either version 2 of the License, or (at your option)
  * any later version.
- * 
+ *
  * playlist.c is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with playlist.c.  If not, write to:
  * 	The Free Software Foundation, Inc.,
@@ -27,7 +27,6 @@
 #include "gui.h"
 #include "support.h"
 
-
 void update_gui()
 {
     if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL) < 2) {
@@ -50,6 +49,9 @@ void update_gui()
         gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), FALSE);
     }
 
+    if (use_defaultpl && safe_to_save_default_playlist)
+        save_playlist_pls(default_playlist);
+
 }
 
 gboolean playlist_popup_handler(GtkWidget * widget, GdkEvent * event, void *data)
@@ -192,7 +194,7 @@ gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
 
         if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL) == 1) {
             gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
-            play_iter(&iter);
+            play_iter(&iter, 0);
 
         } else {
             gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_random), TRUE);
@@ -241,6 +243,21 @@ gboolean playlist_motion_callback(GtkWidget * widget, GdkEventMotion * event, gp
 
 }
 
+gboolean playlist_enter_callback(GtkWidget * widget, GdkEventMotion * event, gpointer user_data)
+{
+    gtk_tree_view_set_enable_search(GTK_TREE_VIEW(list), TRUE);
+    gtk_tree_view_set_search_column(GTK_TREE_VIEW(list), DESCRIPTION_COLUMN);
+    gtk_widget_grab_focus(list);
+    setup_accelerators(FALSE);
+    return TRUE;
+}
+
+gboolean playlist_leave_callback(GtkWidget * widget, GdkEventMotion * event, gpointer user_data)
+{
+    gtk_tree_view_set_enable_search(GTK_TREE_VIEW(list), FALSE);
+    setup_accelerators(TRUE);
+    return TRUE;
+}
 
 void save_playlist(GtkWidget * widget, void *data)
 {
@@ -343,7 +360,6 @@ void load_playlist(GtkWidget * widget, void *data)
         g_free(last_dir);
 
         gtk_list_store_clear(playliststore);
-        gtk_list_store_clear(nonrandomplayliststore);
 
         create_folder_progress_window();
         if (!parse_playlist(filename)) {
@@ -472,6 +488,8 @@ void add_to_playlist(GtkWidget * widget, void *data)
     GtkTreeViewColumn *column;
     gchar *coltitle;
     gint count;
+    gchar **split;
+    gchar *joined;
 
     dialog = gtk_file_chooser_dialog_new(_("Open File"),
                                          GTK_WINDOW(window),
@@ -513,8 +531,13 @@ void add_to_playlist(GtkWidget * widget, void *data)
     } else {
         coltitle = g_strdup_printf(ngettext("Item to Play", "Items to Play", count));
     }
-    gtk_tree_view_column_set_title(column, coltitle);
+
+    split = g_strsplit(coltitle, "_", 0);
+    joined = g_strjoinv("__", split);
+    gtk_tree_view_column_set_title(column, joined);
     g_free(coltitle);
+    g_free(joined);
+    g_strfreev(split);
 
     gtk_widget_destroy(dialog);
 }
@@ -579,28 +602,13 @@ void remove_from_playlist(GtkWidget * widget, gpointer data)
     GtkTreeView *view = (GtkTreeView *) data;
     GtkTreeIter localiter;
     GtkTreePath *path;
-    gchar *localfilename;
-    gchar *removedfilename;
 
     sel = gtk_tree_view_get_selection(view);
 
     if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
-        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &removedfilename, -1);
         localiter = iter;
         gtk_tree_model_iter_next(GTK_TREE_MODEL(playliststore), &localiter);
         if (gtk_list_store_remove(playliststore, &iter)) {
-            iter = localiter;
-            gtk_tree_model_get_iter_first(GTK_TREE_MODEL(nonrandomplayliststore), &localiter);
-            do {
-                gtk_tree_model_get(GTK_TREE_MODEL(nonrandomplayliststore), &localiter, ITEM_COLUMN,
-                                   &localfilename, -1);
-                if (localfilename != NULL) {
-                    if (g_ascii_strcasecmp(removedfilename, localfilename) == 0) {
-                        gtk_list_store_remove(nonrandomplayliststore, &localiter);
-                    }
-                    g_free(localfilename);
-                }
-            } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(nonrandomplayliststore), &localiter));
 
             if (!gtk_list_store_iter_is_valid(playliststore, &iter)) {
                 gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
@@ -613,7 +621,6 @@ void remove_from_playlist(GtkWidget * widget, gpointer data)
                 gtk_tree_path_free(path);
             }
         }
-        g_free(removedfilename);
     }
     update_gui();
 
@@ -625,7 +632,6 @@ void clear_playlist(GtkWidget * widget, void *data)
     dontplaynext = TRUE;
     mplayer_shutdown();
     gtk_list_store_clear(playliststore);
-    gtk_list_store_clear(nonrandomplayliststore);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_edit_random), FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_random), FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), FALSE);
@@ -638,6 +644,7 @@ void move_item_up(GtkWidget * widget, void *data)
     GtkTreePath *path;
     gint pos = 0;
     gchar *buf;
+    gint a_order, b_order;
 
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
     if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
@@ -651,13 +658,12 @@ void move_item_up(GtkWidget * widget, void *data)
             if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &a, NULL, pos)) {
                 if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &b, NULL, pos - 1)) {
                     gtk_list_store_swap(playliststore, &a, &b);
-                }
-            }
-            if (gtk_tree_model_iter_nth_child
-                (GTK_TREE_MODEL(nonrandomplayliststore), &a, NULL, pos)) {
-                if (gtk_tree_model_iter_nth_child
-                    (GTK_TREE_MODEL(nonrandomplayliststore), &b, NULL, pos - 1)) {
-                    gtk_list_store_swap(nonrandomplayliststore, &a, &b);
+                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &a, PLAY_ORDER_COLUMN,
+                                       &a_order, -1);
+                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &b, PLAY_ORDER_COLUMN,
+                                       &b_order, -1);
+                    gtk_list_store_set(playliststore, &a, PLAY_ORDER_COLUMN, b_order, -1);
+                    gtk_list_store_set(playliststore, &b, PLAY_ORDER_COLUMN, a_order, -1);
                 }
             }
         }
@@ -677,6 +683,7 @@ void move_item_down(GtkWidget * widget, void *data)
     GtkTreePath *path;
     gint pos = 0;
     gchar *buf;
+    gint a_order, b_order;
 
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
     if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
@@ -689,13 +696,13 @@ void move_item_down(GtkWidget * widget, void *data)
             if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &a, NULL, pos)) {
                 if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &b, NULL, pos + 1)) {
                     gtk_list_store_swap(playliststore, &a, &b);
-                }
-            }
-            if (gtk_tree_model_iter_nth_child
-                (GTK_TREE_MODEL(nonrandomplayliststore), &a, NULL, pos)) {
-                if (gtk_tree_model_iter_nth_child
-                    (GTK_TREE_MODEL(nonrandomplayliststore), &b, NULL, pos + 1)) {
-                    gtk_list_store_swap(nonrandomplayliststore, &a, &b);
+                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &a, PLAY_ORDER_COLUMN,
+                                       &a_order, -1);
+                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &b, PLAY_ORDER_COLUMN,
+                                       &b_order, -1);
+                    gtk_list_store_set(playliststore, &a, PLAY_ORDER_COLUMN, b_order, -1);
+                    gtk_list_store_set(playliststore, &b, PLAY_ORDER_COLUMN, a_order, -1);
+
                 }
             }
         }
@@ -716,7 +723,7 @@ gboolean playlist_select_callback(GtkTreeView * view, GtkTreePath * path,
     if (gtk_tree_model_get_iter(GTK_TREE_MODEL(playliststore), &iter, path)) {
         dontplaynext = TRUE;
         mplayer_shutdown();
-        play_iter(&iter);
+        play_iter(&iter, 0);
         dontplaynext = FALSE;
     }
     return FALSE;
@@ -729,8 +736,18 @@ void playlist_close(GtkWidget * widget, void *data)
 
 void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
 {
+    playlist_visible = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem));
+    adjust_layout();
+}
 
-    GtkWidget *close;
+void undo_playlist_sort(GtkWidget * widget, void *data)
+{
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(playliststore), PLAY_ORDER_COLUMN,
+                                         GTK_SORT_ASCENDING);
+}
+
+void create_playlist_widget()
+{
     GtkWidget *scrolled;
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
@@ -744,306 +761,266 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
     GtkWidget *remove;
     GtkWidget *add_folder;
     GtkWidget *clear;
+    GtkWidget *undo;
     GtkTreePath *path;
     GtkAccelGroup *accel_group;
-    GValue value = { 0, };
     GtkTooltips *tooltip;
     gchar *coltitle;
-    gint x, y, depth;
     gint count;
     GtkTargetEntry target_entry[3];
     gint i = 0;
     gint handle_size;
+    gchar **split;
+    gchar *joined;
 
-    g_value_init(&value, G_TYPE_BOOLEAN);
+    plvbox = gtk_vbox_new(FALSE, 12);
+    hbox = gtk_hbox_new(FALSE, 12);
+    gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
+    box = gtk_hbox_new(FALSE, 10);
+    ctrlbox = gtk_hbox_new(FALSE, 0);
+    closebox = gtk_hbox_new(FALSE, 0);
 
-    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem_view_playlist)) == FALSE) {
-        if (GTK_WIDGET_VISIBLE(plvbox)) {
-            if (idledata->videopresent == FALSE) {
-                window_width = -1;
-                window_height = -1;
-                gtk_window_get_size(GTK_WINDOW(window), &window_width, &window_height);
-                gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
-                gtk_widget_hide(GTK_WIDGET(fixed));
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_info), TRUE);
-                gtk_widget_set_size_request(window, -1, -1);
-            } else {
-                gtk_widget_style_get(pane, "handle-size", &handle_size, NULL);
-                gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
-                gtk_widget_set_size_request(window, -1, -1);
-                gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
-                window_width = -1;
-                window_height = -1;
-                gtk_window_get_size(GTK_WINDOW(window), &window_width, &window_height);
-                gtk_widget_hide(plvbox);
-                if (vertical_layout) {
-                    gtk_window_resize(GTK_WINDOW(window), window_width,
-                                      window_height - plvbox->allocation.height - handle_size);
-                } else {
-                    gtk_window_resize(GTK_WINDOW(window),
-                                      window_width - plvbox->allocation.width - handle_size,
-                                      window_height);
-                }
-            }
-            gtk_container_remove(GTK_CONTAINER(pane), plvbox);
-            stored_window_width = -1;
-            stored_window_height = -1;
-            plvbox = NULL;
-            selection = NULL;
-            playlist_visible = FALSE;
-        }
+    list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(playliststore));
+    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(list), FALSE);
+    gtk_widget_add_events(list, GDK_BUTTON_PRESS_MASK);
+    gtk_widget_set_size_request(GTK_WIDGET(list), -1, -1);
 
-    } else {
-        gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
-        if (idledata->videopresent == FALSE) {
-            if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem_view_info))) {
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_info), TRUE);
-            }
-        }
-        if (restore_playlist) {
-            if (vertical_layout) {
-                stored_window_height = restore_pane;
-            } else {
-                stored_window_width = restore_pane;
-            }
-            restore_playlist = FALSE;
-        } else {
-            gdk_window_get_geometry(window->window, &x, &y, &stored_window_width,
-                                    &stored_window_height, &depth);
-        }
-        plvbox = gtk_vbox_new(FALSE, 12);
-        hbox = gtk_hbox_new(FALSE, 12);
-        gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
-        box = gtk_hbox_new(FALSE, 10);
-        ctrlbox = gtk_hbox_new(FALSE, 0);
-        closebox = gtk_hbox_new(FALSE, 0);
-
-        list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(playliststore));
-        gtk_tree_view_set_reorderable(GTK_TREE_VIEW(list), FALSE);
-        gtk_widget_add_events(list, GDK_BUTTON_PRESS_MASK);
-        gtk_widget_set_size_request(GTK_WIDGET(list), -1, -1);
-
-        selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
-        if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
-            path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
-            gtk_tree_selection_select_path(selection, path);
-            gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
-            gtk_tree_path_free(path);
-        }
-
-        g_signal_connect(G_OBJECT(list), "row_activated", G_CALLBACK(playlist_select_callback),
-                         NULL);
-        g_signal_connect(G_OBJECT(list), "button-release-event",
-                         G_CALLBACK(button_release_callback), NULL);
-        g_signal_connect(G_OBJECT(list), "motion-notify-event",
-                         G_CALLBACK(playlist_motion_callback), NULL);
-        // Give the window the property to accept DnD
-        target_entry[i].target = DRAG_NAME_0;
-        target_entry[i].flags = 0;
-        target_entry[i++].info = DRAG_INFO_0;
-        target_entry[i].target = DRAG_NAME_1;
-        target_entry[i].flags = 0;
-        target_entry[i++].info = DRAG_INFO_1;
-        target_entry[i].target = DRAG_NAME_2;
-        target_entry[i].flags = 0;
-        target_entry[i++].info = DRAG_INFO_2;
-
-        gtk_drag_dest_set(plvbox,
-                          GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT |
-                          GTK_DEST_DEFAULT_DROP, target_entry, i, GDK_ACTION_LINK);
-
-        //Connect the signal for DnD
-        g_signal_connect(GTK_OBJECT(plvbox), "drag_data_received",
-                         GTK_SIGNAL_FUNC(playlist_drop_callback), NULL);
-
-
-        playlisttip = gtk_tooltips_new();
-
-        count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
-        renderer = gtk_cell_renderer_text_new();
-        column =
-            gtk_tree_view_column_new_with_attributes(ngettext
-                                                     ("Item to play", "Items to Play", count),
-                                                     renderer, "text", DESCRIPTION_COLUMN, NULL);
-        if (playlistname != NULL) {
-            coltitle = g_strdup_printf(_("%s items"), playlistname);
-            gtk_tree_view_column_set_title(column, coltitle);
-            g_free(coltitle);
-        }
-        gtk_tree_view_column_set_expand(column, TRUE);
-        //gtk_tree_view_column_set_max_width(column, 40);
-        g_object_set(renderer, "width-chars", 40, NULL);
-        gtk_tree_view_column_set_resizable(column, TRUE);
-        gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
-
-        renderer = gtk_cell_renderer_text_new();
-        column = gtk_tree_view_column_new_with_attributes(_("Artist"),
-                                                          renderer, "text", ARTIST_COLUMN, NULL);
-        gtk_tree_view_column_set_expand(column, TRUE);
-        //gtk_tree_view_column_set_max_width(column, 20);
-        g_object_set(renderer, "width-chars", 20, NULL);
-        gtk_tree_view_column_set_alignment(column, 0.0);
-        gtk_tree_view_column_set_resizable(column, TRUE);
-        gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
-
-        renderer = gtk_cell_renderer_text_new();
-        column = gtk_tree_view_column_new_with_attributes(_("Album"),
-                                                          renderer, "text", ALBUM_COLUMN, NULL);
-        gtk_tree_view_column_set_expand(column, TRUE);
-        //gtk_tree_view_column_set_max_width(column, 20);
-        g_object_set(renderer, "width-chars", 20, NULL);
-        gtk_tree_view_column_set_alignment(column, 0.0);
-        gtk_tree_view_column_set_resizable(column, TRUE);
-        gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
-
-
-        renderer = gtk_cell_renderer_text_new();
-        column = gtk_tree_view_column_new_with_attributes(_("Length"),
-                                                          renderer, "text", LENGTH_COLUMN, NULL);
-        //gtk_tree_view_column_set_expand(column, FALSE);
-        gtk_tree_view_column_set_alignment(column, 1.0);
-        g_object_set(renderer, "xalign", 1.0, NULL);
-        gtk_tree_view_column_set_resizable(column, FALSE);
-        gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
-
-
-        renderer = gtk_cell_renderer_text_new();
-        column = gtk_tree_view_column_new_with_attributes("", renderer, "text", COUNT_COLUMN, NULL);
-        //gtk_tree_view_column_set_expand(column, FALSE);
-        g_object_set(renderer, "xalign", 1.0, NULL);
-        gtk_tree_view_column_set_resizable(column, FALSE);
-        gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
-
-
-        close = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, close, _("Close Playlist View"), NULL);
-        gtk_container_add(GTK_CONTAINER(close),
-                          gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
-
-        g_signal_connect_swapped(GTK_OBJECT(close), "clicked",
-                                 GTK_SIGNAL_FUNC(playlist_close), NULL);
-
-
-        loadlist = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, loadlist, _("Open Playlist"), NULL);
-        gtk_container_add(GTK_CONTAINER(loadlist),
-                          gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), loadlist, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(loadlist), "clicked", GTK_SIGNAL_FUNC(load_playlist), NULL);
-
-        savelist = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, savelist, _("Save Playlist"), NULL);
-        gtk_container_add(GTK_CONTAINER(savelist),
-                          gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), savelist, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(savelist), "clicked", GTK_SIGNAL_FUNC(save_playlist), NULL);
-
-        add = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, add, _("Add Item to Playlist"), NULL);
-        gtk_button_set_image(GTK_BUTTON(add),
-                             gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), add, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(add), "clicked", GTK_SIGNAL_FUNC(add_to_playlist), NULL);
-
-        remove = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, remove, _("Remove Item from Playlist"), NULL);
-        gtk_button_set_image(GTK_BUTTON(remove),
-                             gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), remove, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(remove), "clicked", GTK_SIGNAL_FUNC(remove_from_playlist),
-                         list);
-
-        add_folder = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, add_folder, _("Add Items in Folder to Playlist"), NULL);
-        gtk_button_set_image(GTK_BUTTON(add_folder),
-                             gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), add_folder, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(add_folder), "clicked", GTK_SIGNAL_FUNC(add_folder_to_playlist),
-                         list);
-
-        clear = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, clear, _("Clear Playlist"), NULL);
-        gtk_button_set_image(GTK_BUTTON(clear),
-                             gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), clear, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(clear), "clicked", GTK_SIGNAL_FUNC(clear_playlist), list);
-
-        up = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, up, _("Move Item Up"), NULL);
-        gtk_button_set_image(GTK_BUTTON(up),
-                             gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), up, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(up), "clicked", GTK_SIGNAL_FUNC(move_item_up), list);
-        gtk_widget_set_sensitive(up, FALSE);
-
-        down = gtk_button_new();
-        tooltip = gtk_tooltips_new();
-        gtk_tooltips_set_tip(tooltip, down, _("Move Item Down"), NULL);
-        gtk_button_set_image(GTK_BUTTON(down),
-                             gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU));
-        gtk_box_pack_start(GTK_BOX(ctrlbox), down, FALSE, FALSE, 0);
-        g_signal_connect(GTK_OBJECT(down), "clicked", GTK_SIGNAL_FUNC(move_item_down), list);
-        gtk_widget_set_sensitive(down, FALSE);
-
-
-        accel_group = gtk_accel_group_new();
-        gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
-        gtk_widget_add_accelerator(GTK_WIDGET(remove), "clicked",
-                                   accel_group, GDK_Delete, 0, GTK_ACCEL_VISIBLE);
-
-        GTK_WIDGET_SET_FLAGS(close, GTK_CAN_DEFAULT);
-        gtk_box_pack_end(GTK_BOX(closebox), close, FALSE, FALSE, 0);
-
-        scrolled = gtk_scrolled_window_new(NULL, NULL);
-        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-        gtk_container_add(GTK_CONTAINER(scrolled), list);
-
-        gtk_box_pack_start(GTK_BOX(plvbox), scrolled, TRUE, TRUE, 0);
-        gtk_box_pack_start(GTK_BOX(hbox), ctrlbox, FALSE, FALSE, 0);
-        gtk_box_pack_end(GTK_BOX(hbox), closebox, FALSE, FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(plvbox), hbox, FALSE, FALSE, 0);
-
-
-        gtk_paned_pack2(GTK_PANED(pane), plvbox, TRUE, TRUE);
-        adjust_paned_rules();
-
-        gtk_widget_style_get(pane, "handle-size", &handle_size, NULL);
-        if (vertical_layout) {
-            gtk_widget_set_size_request(plvbox, -1, 150);
-            gtk_window_resize(GTK_WINDOW(window), stored_window_width,
-                              stored_window_height + 150 + handle_size);
-        } else {
-            gtk_widget_set_size_request(plvbox, 300, -1);
-            gtk_window_resize(GTK_WINDOW(window), stored_window_width + 300 + handle_size,
-                              stored_window_height);
-        }
-        gtk_widget_show_all(plvbox);
-        if (idledata->videopresent) {
-            gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
-        }
-
-        playlist_popup_menu = GTK_MENU(gtk_menu_new());
-        playlist_set_subtitle =
-            GTK_MENU_ITEM(gtk_image_menu_item_new_with_mnemonic(_("_Set Subtitle")));
-        g_signal_connect(GTK_OBJECT(playlist_set_subtitle), "activate",
-                         G_CALLBACK(playlist_set_subtitle_callback), list);
-
-        gtk_menu_append(playlist_popup_menu, GTK_WIDGET(playlist_set_subtitle));
-        g_signal_connect_swapped(G_OBJECT(list),
-                                 "button_press_event",
-                                 G_CALLBACK(playlist_popup_handler), G_OBJECT(playlist_popup_menu));
-        gtk_widget_show_all(GTK_WIDGET(playlist_popup_menu));
-        playlist_visible = TRUE;
-
-        gtk_widget_grab_default(close);
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+    if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+        path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
+        gtk_tree_selection_select_path(selection, path);
+        gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
+        gtk_tree_path_free(path);
     }
+
+    g_signal_connect(G_OBJECT(list), "row_activated", G_CALLBACK(playlist_select_callback), NULL);
+    g_signal_connect(G_OBJECT(list), "button-release-event",
+                     G_CALLBACK(button_release_callback), NULL);
+    g_signal_connect(G_OBJECT(list), "motion-notify-event",
+                     G_CALLBACK(playlist_motion_callback), NULL);
+    g_signal_connect(G_OBJECT(list), "enter-notify-event",
+                     G_CALLBACK(playlist_enter_callback), NULL);
+    g_signal_connect(G_OBJECT(list), "leave-notify-event",
+                     G_CALLBACK(playlist_leave_callback), NULL);
+    // Give the window the property to accept DnD
+    target_entry[i].target = DRAG_NAME_0;
+    target_entry[i].flags = 0;
+    target_entry[i++].info = DRAG_INFO_0;
+    target_entry[i].target = DRAG_NAME_1;
+    target_entry[i].flags = 0;
+    target_entry[i++].info = DRAG_INFO_1;
+    target_entry[i].target = DRAG_NAME_2;
+    target_entry[i].flags = 0;
+    target_entry[i++].info = DRAG_INFO_2;
+
+    gtk_drag_dest_set(plvbox,
+                      GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT |
+                      GTK_DEST_DEFAULT_DROP, target_entry, i, GDK_ACTION_LINK);
+
+    //Connect the signal for DnD
+    g_signal_connect(GTK_OBJECT(plvbox), "drag_data_received",
+                     GTK_SIGNAL_FUNC(playlist_drop_callback), NULL);
+
+
+    playlisttip = gtk_tooltips_new();
+
+    count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
+    renderer = gtk_cell_renderer_text_new();
+    column =
+        gtk_tree_view_column_new_with_attributes(ngettext
+                                                 ("Item to play", "Items to Play", count),
+                                                 renderer, "text", DESCRIPTION_COLUMN, NULL);
+    if (playlistname != NULL) {
+        coltitle = g_strdup_printf(_("%s items"), playlistname);
+        split = g_strsplit(coltitle, "_", 0);
+        joined = g_strjoinv("__", split);
+        gtk_tree_view_column_set_title(column, joined);
+        g_free(coltitle);
+        g_free(joined);
+        g_strfreev(split);
+    }
+    gtk_tree_view_column_set_expand(column, TRUE);
+    //gtk_tree_view_column_set_max_width(column, 40);
+    g_object_set(renderer, "width-chars", 40, NULL);
+    gtk_tree_view_column_set_resizable(column, TRUE);
+    gtk_tree_view_column_set_sort_column_id(column, DESCRIPTION_COLUMN);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes(_("Artist"),
+                                                      renderer, "text", ARTIST_COLUMN, NULL);
+    gtk_tree_view_column_set_expand(column, TRUE);
+    //gtk_tree_view_column_set_max_width(column, 20);
+    g_object_set(renderer, "width-chars", 20, NULL);
+    gtk_tree_view_column_set_alignment(column, 0.0);
+    gtk_tree_view_column_set_resizable(column, TRUE);
+    gtk_tree_view_column_set_sort_column_id(column, ARTIST_COLUMN);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes(_("Album"),
+                                                      renderer, "text", ALBUM_COLUMN, NULL);
+    gtk_tree_view_column_set_expand(column, TRUE);
+    //gtk_tree_view_column_set_max_width(column, 20);
+    g_object_set(renderer, "width-chars", 20, NULL);
+    gtk_tree_view_column_set_alignment(column, 0.0);
+    gtk_tree_view_column_set_resizable(column, TRUE);
+    gtk_tree_view_column_set_sort_column_id(column, ALBUM_COLUMN);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes(_("Length"),
+                                                      renderer, "text", LENGTH_COLUMN, NULL);
+    //gtk_tree_view_column_set_expand(column, FALSE);
+    gtk_tree_view_column_set_alignment(column, 1.0);
+    g_object_set(renderer, "xalign", 1.0, NULL);
+    gtk_tree_view_column_set_resizable(column, FALSE);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+
+    renderer = gtk_cell_renderer_text_new();
+    column =
+        gtk_tree_view_column_new_with_attributes(_("Count"), renderer, "text", COUNT_COLUMN, NULL);
+    //gtk_tree_view_column_set_expand(column, FALSE);
+    g_object_set(renderer, "xalign", 1.0, NULL);
+    gtk_tree_view_column_set_resizable(column, FALSE);
+    gtk_tree_view_column_set_sort_column_id(column, COUNT_COLUMN);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+    renderer = gtk_cell_renderer_text_new();
+    column =
+        gtk_tree_view_column_new_with_attributes(_("Order"), renderer, "text", PLAY_ORDER_COLUMN,
+                                                 NULL);
+    //gtk_tree_view_column_set_expand(column, FALSE);
+    g_object_set(renderer, "xalign", 1.0, NULL);
+    gtk_tree_view_column_set_resizable(column, FALSE);
+    gtk_tree_view_column_set_sort_column_id(column, PLAY_ORDER_COLUMN);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+
+    plclose = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, plclose, _("Close Playlist View"), NULL);
+    gtk_container_add(GTK_CONTAINER(plclose),
+                      gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
+
+    g_signal_connect_swapped(GTK_OBJECT(plclose), "clicked", GTK_SIGNAL_FUNC(playlist_close), NULL);
+
+
+    loadlist = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, loadlist, _("Open Playlist"), NULL);
+    gtk_container_add(GTK_CONTAINER(loadlist),
+                      gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), loadlist, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(loadlist), "clicked", GTK_SIGNAL_FUNC(load_playlist), NULL);
+
+    savelist = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, savelist, _("Save Playlist"), NULL);
+    gtk_container_add(GTK_CONTAINER(savelist),
+                      gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), savelist, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(savelist), "clicked", GTK_SIGNAL_FUNC(save_playlist), NULL);
+
+    add = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, add, _("Add Item to Playlist"), NULL);
+    gtk_button_set_image(GTK_BUTTON(add),
+                         gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), add, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(add), "clicked", GTK_SIGNAL_FUNC(add_to_playlist), NULL);
+
+    remove = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, remove, _("Remove Item from Playlist"), NULL);
+    gtk_button_set_image(GTK_BUTTON(remove),
+                         gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), remove, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(remove), "clicked", GTK_SIGNAL_FUNC(remove_from_playlist), list);
+
+    add_folder = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, add_folder, _("Add Items in Folder to Playlist"), NULL);
+    gtk_button_set_image(GTK_BUTTON(add_folder),
+                         gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), add_folder, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(add_folder), "clicked", GTK_SIGNAL_FUNC(add_folder_to_playlist),
+                     list);
+
+    clear = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, clear, _("Clear Playlist"), NULL);
+    gtk_button_set_image(GTK_BUTTON(clear),
+                         gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), clear, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(clear), "clicked", GTK_SIGNAL_FUNC(clear_playlist), list);
+
+    up = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, up, _("Move Item Up"), NULL);
+    gtk_button_set_image(GTK_BUTTON(up),
+                         gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), up, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(up), "clicked", GTK_SIGNAL_FUNC(move_item_up), list);
+    gtk_widget_set_sensitive(up, FALSE);
+
+    down = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, down, _("Move Item Down"), NULL);
+    gtk_button_set_image(GTK_BUTTON(down),
+                         gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), down, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(down), "clicked", GTK_SIGNAL_FUNC(move_item_down), list);
+    gtk_widget_set_sensitive(down, FALSE);
+
+
+    undo = gtk_button_new();
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, undo, _("UnSort List"), NULL);
+    gtk_button_set_image(GTK_BUTTON(undo),
+                         gtk_image_new_from_stock(GTK_STOCK_UNDO, GTK_ICON_SIZE_MENU));
+    gtk_box_pack_start(GTK_BOX(ctrlbox), undo, FALSE, FALSE, 0);
+    g_signal_connect(GTK_OBJECT(undo), "clicked", GTK_SIGNAL_FUNC(undo_playlist_sort), list);
+    gtk_widget_set_sensitive(undo, TRUE);
+
+
+    accel_group = gtk_accel_group_new();
+    gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
+    gtk_widget_add_accelerator(GTK_WIDGET(remove), "clicked",
+                               accel_group, GDK_Delete, 0, GTK_ACCEL_VISIBLE);
+
+    GTK_WIDGET_SET_FLAGS(plclose, GTK_CAN_DEFAULT);
+    gtk_box_pack_end(GTK_BOX(closebox), plclose, FALSE, FALSE, 0);
+
+    scrolled = gtk_scrolled_window_new(NULL, NULL);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+    gtk_container_add(GTK_CONTAINER(scrolled), list);
+
+    gtk_box_pack_start(GTK_BOX(plvbox), scrolled, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), ctrlbox, FALSE, FALSE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), closebox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(plvbox), hbox, FALSE, FALSE, 0);
+
+
+    gtk_paned_pack2(GTK_PANED(pane), plvbox, FALSE, FALSE);
+
+    gtk_widget_style_get(pane, "handle-size", &handle_size, NULL);
+    gtk_widget_set_size_request(plvbox, 300, 200);
+
+    playlist_popup_menu = GTK_MENU(gtk_menu_new());
+    playlist_set_subtitle =
+        GTK_MENU_ITEM(gtk_image_menu_item_new_with_mnemonic(_("_Set Subtitle")));
+    g_signal_connect(GTK_OBJECT(playlist_set_subtitle), "activate",
+                     G_CALLBACK(playlist_set_subtitle_callback), list);
+
+    gtk_menu_append(playlist_popup_menu, GTK_WIDGET(playlist_set_subtitle));
+    g_signal_connect_swapped(G_OBJECT(list),
+                             "button_press_event",
+                             G_CALLBACK(playlist_popup_handler), G_OBJECT(playlist_popup_menu));
+    gtk_widget_show_all(GTK_WIDGET(playlist_popup_menu));
+
+
 }
