@@ -38,15 +38,33 @@ void mplayer_shutdown()
             g_idle_add(unmap_af_export_file, idledata);
         }
         send_command("quit\n", FALSE);
+        dbus_send_event("Ended", 0);
     }
 
 }
 
+gboolean write_to_mplayer(gpointer data)
+{
+    gchar *cmd = (gchar *) data;
+    GIOStatus result;
+    gsize bytes_written;
+
+    if (verbose > 1)
+        printf("send command = %s\n", cmd);
+
+    if (channel_in) {
+        result = g_io_channel_write_chars(channel_in, cmd, -1, &bytes_written, NULL);
+        if (result != G_IO_STATUS_ERROR && bytes_written > 0)
+            result = g_io_channel_flush(channel_in, NULL);
+    }
+
+    g_free(cmd);
+    return FALSE;
+}
+
 gboolean send_command(gchar * command, gboolean retain_pause)
 {
     gchar *cmd;
-    GIOStatus result;
-    gsize bytes_written;
 
     if (retain_pause) {
         if (use_pausing_keep_force) {
@@ -58,15 +76,7 @@ gboolean send_command(gchar * command, gboolean retain_pause)
         cmd = g_strdup(command);
     }
 
-    if (verbose > 1)
-        printf("send command = %s\n", cmd);
-
-    if (channel_in) {
-        result = g_io_channel_write_chars(channel_in, cmd, -1, &bytes_written, NULL);
-        result = g_io_channel_flush(channel_in, NULL);
-    }
-
-    g_free(cmd);
+    g_idle_add(write_to_mplayer, (gpointer) cmd);
     return TRUE;
 
 }
@@ -76,9 +86,13 @@ gboolean play(void *data)
     PlayData *p = (PlayData *) data;
 
     if (ok_to_play && p != NULL) {
+        if (!gtk_list_store_iter_is_valid(playliststore, &iter)) {
+            // printf("iter is not valid, getting first one\n");
+            gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
+        }
         gtk_list_store_set(playliststore, &iter, PLAYLIST_COLUMN, p->playlist, ITEM_COLUMN, p->uri,
                            -1);
-        play_iter(&iter);
+        play_iter(&iter, 0);
     }
     g_free(p);
 
@@ -87,6 +101,8 @@ gboolean play(void *data)
 
 gboolean thread_complete(GIOChannel * source, GIOCondition condition, gpointer data)
 {
+    //ThreadData *threaddata = (ThreadData *) data;
+
     if (verbose > 1)
         printf("thread complete\n");
     g_idle_add(set_stop, idledata);
@@ -103,6 +119,7 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
     GIOStatus status;
     gchar *error_msg = NULL;
     GtkWidget *dialog;
+    ThreadData *threaddata = (ThreadData *) data;
 
     if (source == NULL) {
         g_source_remove(watch_in_id);
@@ -110,6 +127,21 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         return FALSE;
     }
 
+    if (data == NULL) {
+        if (verbose)
+            printf("shutting down threadquery since threaddata is NULL\n");
+        return FALSE;
+    }
+
+    if (threaddata != NULL && threaddata->done == TRUE) {
+        if (verbose)
+            printf("shutting down threadquery for %s since threaddata->done is TRUE\n",
+                   threaddata->filename);
+        g_free(threaddata);
+        threaddata = NULL;
+        return FALSE;
+    }
+
     mplayer_output = g_string_new("");
     status = g_io_channel_read_line_string(source, mplayer_output, NULL, NULL);
 
@@ -167,7 +199,19 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         playback_error = ERROR_RETRY_WITH_HTTP;
     }
 
-	if (strstr(mplayer_output->str, "Error while parsing chunk header") != NULL) {
+    if (strstr(mplayer_output->str, "Server returned 404:File Not Found") != NULL
+        && g_strrstr(threaddata->filename, "mmshttp://") != NULL) {
+        dontplaynext = TRUE;
+        playback_error = ERROR_RETRY_WITH_HTTP;
+    }
+
+    if (strstr(mplayer_output->str, "unknown ASF streaming type") != NULL
+        && g_strrstr(threaddata->filename, "mmshttp://") != NULL) {
+        dontplaynext = TRUE;
+        playback_error = ERROR_RETRY_WITH_HTTP;
+    }
+
+    if (strstr(mplayer_output->str, "Error while parsing chunk header") != NULL) {
         dontplaynext = TRUE;
         playback_error = ERROR_RETRY_WITH_HTTP;
     }
@@ -205,7 +249,7 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
 
     if (error_msg != NULL) {
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
-                                        GTK_BUTTONS_CLOSE, error_msg);
+                                        GTK_BUTTONS_CLOSE, "%s", error_msg);
         gtk_window_set_title(GTK_WINDOW(dialog), _("GNOME MPlayer Error"));
         if (control_id == 0)
             gtk_dialog_run(GTK_DIALOG(dialog));
@@ -224,6 +268,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     GString *mplayer_output;
     GIOStatus status;
     gchar *buf, *message = NULL;
+    gchar *cmd;
     gint pos, volume, i;
     gfloat percent;
     GError *error = NULL;
@@ -231,6 +276,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     GtkWidget *dialog;
     gdouble old_pos;
     LangMenu *menu;
+    ThreadData *threaddata = (ThreadData *) data;
 
     if (source == NULL) {
         g_source_remove(watch_err_id);
@@ -240,6 +286,21 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
 
     idledata->fromdbus = FALSE;
 
+    if (data == NULL) {
+        if (verbose)
+            printf("shutting down threadquery since threaddata is NULL\n");
+        return FALSE;
+    }
+
+    if (threaddata != NULL && threaddata->done == TRUE) {
+        if (verbose)
+            printf("shutting down threadquery for %s since threaddata->done is TRUE\n",
+                   threaddata->filename);
+        g_free(threaddata);
+        threaddata = NULL;
+        return FALSE;
+    }
+
     if (state == QUIT) {
         if (verbose)
             printf("Thread: state = QUIT, shutting down\n");
@@ -309,29 +370,41 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_string_free(mplayer_output, TRUE);
         //g_mutex_unlock(thread_running);
         g_cond_signal(mplayer_complete_cond);
+
+        if (idledata->position < 0.05 && g_strrstr(threaddata->filename, "http://") != NULL
+            && g_strrstr(threaddata->filename, "mmshttp://") == NULL) {
+            dontplaynext = TRUE;
+            playback_error = ERROR_RETRY_WITH_PLAYLIST;
+        }
+
         return FALSE;
     }
 
     if (strstr(mplayer_output->str, "AO:") != NULL) {
         idledata->audiopresent = TRUE;
+        send_command("get_property switch_audio\n", TRUE);
     }
 
     if (strstr(mplayer_output->str, "VO:") != NULL) {
         buf = strstr(mplayer_output->str, "VO:");
         sscanf(buf, "VO: [%9[^]]] %ix%i => %ix%i", vm, &actual_x, &actual_y, &play_x, &play_y);
 
-        if (play_x >= actual_x && play_y >= actual_y) {
-            if (actual_x == last_window_width && actual_y == last_window_height) {
-                last_window_width = 0;
-                last_window_height = 0;
+        if (play_x >= actual_x || play_y >= actual_y) {
+            if (actual_x == non_fs_width && actual_y == non_fs_height) {
+                non_fs_height = 0;
+                non_fs_width = 0;
             }
             actual_x = play_x;
             actual_y = play_y;
         }
         if (verbose)
-            printf("Resizing to %i x %i \n", actual_x, actual_y);
+            printf("Resizing to %i x %i\n", actual_x, actual_y);
         idledata->width = actual_x;
         idledata->height = actual_y;
+        if (idledata->original_w == -1 && idledata->original_h == -1) {
+            idledata->original_w = idledata->width;
+            idledata->original_h = idledata->height;
+        }
         idledata->videopresent = TRUE;
         g_idle_add(resize_window, idledata);
         g_idle_add(set_subtitle_visibility, idledata);
@@ -340,9 +413,36 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         if (idledata->length < 1.0)
             send_command("get_time_length\n", TRUE);
         send_command("get_property chapters\n", TRUE);
-        send_command("get_property sub_demux\n", TRUE);
         // send_command("get_property switch_audio\n", TRUE);
         send_command("pausing_keep_force get_property path\n", FALSE);
+        if (sub_source_file) {
+            if (verbose)
+                printf("setting subtitle source to file\n");
+            send_command("sub_file 0\n", TRUE);
+        } else {
+            send_command("get_property sub_demux\n", TRUE);
+        }
+
+        cmd = g_strdup_printf("brightness %i\n", idledata->brightness);
+        send_command(cmd, TRUE);
+        g_free(cmd);
+
+        cmd = g_strdup_printf("contrast %i\n", idledata->contrast);
+        send_command(cmd, TRUE);
+        g_free(cmd);
+
+        cmd = g_strdup_printf("gamma %i\n", idledata->gamma);
+        send_command(cmd, TRUE);
+        g_free(cmd);
+
+        cmd = g_strdup_printf("hue %i\n", idledata->hue);
+        send_command(cmd, TRUE);
+        g_free(cmd);
+
+        cmd = g_strdup_printf("saturation %i\n", idledata->saturation);
+        send_command(cmd, TRUE);
+        g_free(cmd);
+
     }
 
     if (strstr(mplayer_output->str, "Video: no video") != NULL) {
@@ -408,12 +508,11 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         buf = strstr(mplayer_output->str, "ANS_volume");
         sscanf(buf, "ANS_volume=%i", &volume);
         if (!idledata->mute) {
-            idledata->volume = volume;
-            idledata->mute = 0;
-            buf = g_strdup_printf(_("Volume %i%%"), volume);
-            g_strlcpy(idledata->vol_tooltip, buf, 128);
-            g_idle_add(set_volume_tip, idledata);
-            g_free(buf);
+            if (use_pulse_flat_volume && !softvol) {
+                // Need to track what the master volume is, gui is updated in make mouse invisible
+                idledata->mplayer_volume = volume;
+                idledata->mute = (volume > 0);
+            }
         }
     }
 
@@ -572,6 +671,16 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_idle_add(set_new_lang_menu, menu);
     }
 
+    if (strstr(mplayer_output->str, "ID_FILE_SUB_ID=") != 0) {
+        menu = g_new0(LangMenu, 1);
+        buf = strstr(mplayer_output->str, "ID_FILE_SUB_ID");
+        sscanf(buf, "ID_FILE_SUB_ID=%i", &menu->value);
+        menu->label = g_strdup_printf("FILE %i", menu->value);
+        menu->value += 9000;
+        g_idle_add(set_new_lang_menu, menu);
+        sub_source_file = TRUE;
+    }
+
     if (strstr(mplayer_output->str, "ID_AID_") != 0) {
         menu = g_new0(LangMenu, 1);
         buf = strstr(mplayer_output->str, "ID_AID_");
@@ -582,6 +691,11 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             buf += strlen("_LANG=");
             menu->label = g_strdup(buf);
             g_idle_add(set_new_audio_menu, menu);
+            if (alang != NULL && g_strrstr(alang, menu->label) != NULL) {
+                buf = g_strdup_printf("switch_audio %i\n", menu->value);
+                send_command(buf, TRUE);
+                g_free(buf);
+            }
         }
     }
 
@@ -619,7 +733,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         buf[12] = '\0';
         message = g_strdup_printf(_("Screenshot saved to '%s'"), buf);
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO,
-                                        GTK_BUTTONS_OK, message);
+                                        GTK_BUTTONS_OK, "%s", message);
         gtk_window_set_title(GTK_WINDOW(dialog), _("GNOME MPlayer Notification"));
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
@@ -635,12 +749,14 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         }
         if (buf != NULL) {
             for (i = 1; i < (int) strlen(buf) - 1; i++) {
-                if (buf[i] == '\'') {
+                if (!strncmp(&buf[i], "\';", 2)) {
                     buf[i] = '\0';
                     break;
                 }
             }
-            message = g_strdup_printf("<small>\n\t<big><b>%s</b></big>\n</small>", buf + 1);
+            if (g_utf8_validate(buf + 1, strlen(buf + 1), 0))
+                message =
+                    g_markup_printf_escaped("<small>\n\t<big><b>%s</b></big>\n</small>", buf + 1);
         }
         if (message) {
             // reset max values in audio meter
@@ -649,7 +765,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             }
             g_strlcpy(idledata->media_info, message, 1024);
             g_free(message);
-            message = g_strdup_printf("\n\t<b>%s</b>\n", buf + 1);
+            message = g_markup_printf_escaped("\n\t<b>%s</b>\n", buf + 1);
             g_strlcpy(idledata->media_notification, message, 1024);
             g_free(message);
             message = NULL;
@@ -664,7 +780,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
 
     if (error_msg != NULL) {
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
-                                        GTK_BUTTONS_OK, error_msg);
+                                        GTK_BUTTONS_OK, "%s", error_msg);
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
         g_free(error_msg);
@@ -697,7 +813,7 @@ gboolean thread_query(gpointer data)
         return FALSE;
     }
 
-    if (threaddata->done == TRUE) {
+    if (threaddata != NULL && threaddata->done == TRUE) {
         if (verbose)
             printf("shutting down threadquery for %s since threaddata->done is TRUE\n",
                    threaddata->filename);
@@ -711,13 +827,19 @@ gboolean thread_query(gpointer data)
         // but don't start polling until meter is visible
         g_idle_add(map_af_export_file, idledata);
     }
-
+    // track volume control
+    if (use_pulse_flat_volume && !softvol) {
+        volume = (gint) get_alsa_volume(FALSE);
+        idledata->volume = volume;
+        if (!idledata->mute)
+            g_idle_add(set_volume, idledata);
+    }
 
     if (state == PLAYING) {
         // size = write(std_in, "get_percent_pos\n", strlen("get_percent_pos\n"));
         size = write(std_in, "get_time_pos\n", strlen("get_time_pos\n"));
         if (size == -1) {
-            mplayer_shutdown();
+            g_idle_add(set_kill_mplayer,NULL);
             return FALSE;
         } else {
 
@@ -753,6 +875,9 @@ gpointer launch_player(gpointer data)
     GError *error;
     // GIOFlags flags;
     GPid pid;
+#ifdef GIO_ENABLED
+    GFile *file;
+#endif
 
     ThreadData *threaddata = (ThreadData *) data;
 
@@ -777,6 +902,7 @@ gpointer launch_player(gpointer data)
     g_strlcpy(idledata->progress_text, "", 1024);
     idledata->volume = volume;
     idledata->length = 0.0;
+    sub_source_file = FALSE;
 
     g_idle_add(set_progress_value, idledata);
     g_idle_add(set_progress_text, idledata);
@@ -798,8 +924,8 @@ gpointer launch_player(gpointer data)
     }
 
     if (vo != NULL && g_ascii_strncasecmp(vo, "xvmc", strlen("xvmc")) == 0) {
-        if (g_strncasecmp(threaddata->filename, "dvd://", strlen("dvd://")) == 0
-            || g_strncasecmp(threaddata->filename, "dvdnav://", strlen("dvdnav://")) == 0) {
+        if (g_ascii_strncasecmp(threaddata->filename, "dvd://", strlen("dvd://")) == 0
+            || g_ascii_strncasecmp(threaddata->filename, "dvdnav://", strlen("dvdnav://")) == 0) {
             argv[arg++] = g_strdup_printf("-vc");
             argv[arg++] = g_strdup_printf("ffmpeg12mc");
         } else {
@@ -838,6 +964,11 @@ gpointer launch_player(gpointer data)
         }
     }
 
+    if (use_hw_audio) {
+        argv[arg++] = g_strdup_printf("-afm");
+        argv[arg++] = g_strdup_printf("hwac3,");
+    }
+
     if (verbose < 2)
         argv[arg++] = g_strdup_printf("-quiet");
     argv[arg++] = g_strdup_printf("-slave");
@@ -849,7 +980,10 @@ gpointer launch_player(gpointer data)
 
     if (use_volume_option) {
         argv[arg++] = g_strdup_printf("-volume");
-        argv[arg++] = g_strdup_printf("%i", (gint) idledata->volume);
+        if (idledata->mute)
+            argv[arg++] = g_strdup_printf("0");
+        else
+            argv[arg++] = g_strdup_printf("%i", (gint) idledata->volume);
     }
 
     if (mixer != NULL && strlen(mixer) > 0) {
@@ -872,13 +1006,20 @@ gpointer launch_player(gpointer data)
     }
 
     argv[arg++] = g_strdup_printf("-noconsolecontrols");
+    argv[arg++] = g_strdup_printf("-noidle");
+
+#if !(SM_INHIBIT || SS_INHIBIT)
+    if (idledata->width > 0 && idledata->height > 0)
+        argv[arg++] = g_strdup_printf("-stop-xscreensaver");
+#endif
+
     argv[arg++] = g_strdup_printf("-osdlevel");
     argv[arg++] = g_strdup_printf("%i", osdlevel);
     if (strcmp(threaddata->filename, "dvdnav://") == 0) {
         argv[arg++] = g_strdup_printf("-mouse-movements");
         argv[arg++] = g_strdup_printf("-nocache");
     } else {
-        if (g_strncasecmp(threaddata->filename, "dvd://", strlen("dvd://")) == 0) {
+        if (g_ascii_strncasecmp(threaddata->filename, "dvd://", strlen("dvd://")) == 0) {
             // argv[arg++] = g_strdup_printf("-nocache");
         } else {
             argv[arg++] = g_strdup_printf("-nomouseinput");
@@ -886,7 +1027,21 @@ gpointer launch_player(gpointer data)
                 argv[arg++] = g_strdup_printf("-cache");
                 argv[arg++] = g_strdup_printf("%i", cache_size);
             } else {
+#ifdef GIO_ENABLED
+                file = g_file_new_for_uri(threaddata->uri);
+                if (file != NULL) {
+                    if (g_file_is_native(file)) {
+                        argv[arg++] = g_strdup_printf("-nocache");
+                    } else {
+                        forcecache = TRUE;
+                        argv[arg++] = g_strdup_printf("-cache");
+                        argv[arg++] = g_strdup_printf("%i", cache_size);
+                    }
+                    g_object_unref(file);
+                }
+#else
                 argv[arg++] = g_strdup_printf("-nocache");
+#endif
             }
         }
     }
@@ -894,6 +1049,21 @@ gpointer launch_player(gpointer data)
     argv[arg++] = g_strdup_printf("-wid");
     player_window = idledata->windowid;
     argv[arg++] = g_strdup_printf("0x%x", player_window);
+#ifdef ENABLE_PANSCAN
+    argv[arg++] = g_strdup_printf("-fs");
+#endif
+    if (threaddata->restart_second > 0) {
+        argv[arg++] = g_strdup_printf("-ss");
+        argv[arg++] = g_strdup_printf("%i", threaddata->restart_second);
+    } else {
+        argv[arg++] = g_strdup_printf("-ss");
+        argv[arg++] = g_strdup_printf("%i", threaddata->start_second);
+    }
+
+    if (threaddata->play_length > 0) {
+        argv[arg++] = g_strdup_printf("-endpos");
+        argv[arg++] = g_strdup_printf("%i", threaddata->play_length);
+    }
 
     if (control_id == 0) {
 //        if (!idledata->streaming)
@@ -935,34 +1105,41 @@ gpointer launch_player(gpointer data)
     if (!disable_ass) {
         argv[arg++] = g_strdup_printf("-ass");
 
+        if (subtitle_margin > 0) {
+            argv[arg++] = g_strdup_printf("-ass-bottom-margin");
+            argv[arg++] = g_strdup_printf("%i", subtitle_margin);
+            argv[arg++] = g_strdup_printf("-ass-use-margins");
+        }
         // Simply ommiting '-embeddedfonts' did not work
-        if (disable_embeddedfonts)
-            argv[arg++] = g_strdup_printf("-noembeddedfonts");
-        else
+        if (!disable_embeddedfonts && g_strrstr(idledata->demuxer, "mkv")) {
             argv[arg++] = g_strdup_printf("-embeddedfonts");
+        } else {
+            argv[arg++] = g_strdup_printf("-noembeddedfonts");
+
+            if (subtitlefont != NULL && strlen(subtitlefont) > 0) {
+                fontname = g_strdup(subtitlefont);
+                size = g_strrstr(fontname, " ");
+                size[0] = '\0';
+                size = g_strrstr(fontname, " Bold");
+                if (size)
+                    size[0] = '\0';
+                size = g_strrstr(fontname, " Italic");
+                if (size)
+                    size[0] = '\0';
+                argv[arg++] = g_strdup_printf("-ass-force-style");
+                argv[arg++] = g_strconcat("FontName=", fontname,
+                                          ((g_strrstr(subtitlefont, "Italic") !=
+                                            NULL) ? ",Italic=1" : ",Italic=0"),
+                                          ((g_strrstr(subtitlefont, "Bold") !=
+                                            NULL) ? ",Bold=1" : ",Bold=0"),
+                                          (subtitle_outline ? ",Outline=1" : ",Outline=0"),
+                                          (subtitle_shadow ? ",Shadow=2" : ",Shadow=0"), NULL);
+                g_free(fontname);
+            }
+        }
 
         argv[arg++] = g_strdup_printf("-ass-font-scale");
         argv[arg++] = g_strdup_printf("%1.2f", subtitle_scale);
-        if (disable_embeddedfonts && subtitlefont != NULL && strlen(subtitlefont) > 0) {
-            fontname = g_strdup(subtitlefont);
-            size = g_strrstr(fontname, " ");
-            size[0] = '\0';
-            size = g_strrstr(fontname, " Bold");
-            if (size)
-                size[0] = '\0';
-            size = g_strrstr(fontname, " Italic");
-            if (size)
-                size[0] = '\0';
-            argv[arg++] = g_strdup_printf("-ass-force-style");
-            argv[arg++] = g_strconcat("FontName=", fontname,
-                                      ((g_strrstr(subtitlefont, "Italic") !=
-                                        NULL) ? ",Italic=1" : ",Italic=0"),
-                                      ((g_strrstr(subtitlefont, "Bold") !=
-                                        NULL) ? ",Bold=1" : ",Bold=0"),
-                                      (subtitle_outline ? ",Outline=1" : ",Outline=0"),
-                                      (subtitle_shadow ? ",Shadow=2" : ",Shadow=0"), NULL);
-            g_free(fontname);
-        }
 
         if (subtitle_color != NULL && strlen(subtitle_color) > 0) {
             argv[arg++] = g_strdup_printf("-ass-color");
@@ -981,6 +1158,8 @@ gpointer launch_player(gpointer data)
             argv[arg++] = g_strdup_printf("%s", fontname);
             g_free(fontname);
         }
+
+
     }
 
     if (subtitle_codepage != NULL && strlen(subtitle_codepage) > 0) {
@@ -988,6 +1167,19 @@ gpointer launch_player(gpointer data)
         argv[arg++] = g_strdup_printf("%s", subtitle_codepage);
     }
 
+    argv[arg++] = g_strdup_printf("-channels");
+    switch (audio_channels) {
+    case 1:
+        argv[arg++] = g_strdup_printf("4");
+        break;
+    case 2:
+        argv[arg++] = g_strdup_printf("6");
+        break;
+    default:
+        argv[arg++] = g_strdup_printf("2");
+        break;
+    }
+
     if (vo == NULL || !(g_ascii_strncasecmp(vo, "xvmc", strlen("xvmc")) == 0
                         || g_ascii_strncasecmp(vo, "vdpau", strlen("vdpau")) == 0)) {
 
@@ -1025,7 +1217,7 @@ gpointer launch_player(gpointer data)
        http://lists.mplayerhq.hu/pipermail/mplayer-users/2007-February/065316.html 
      */
     filename = g_utf8_strdown(threaddata->filename, -1);
-    if (strstr(filename, ".ogm")) {
+    if (strstr(filename, ".ogm") || strstr(filename, ".ogv")) {
         argv[arg++] = g_strdup_printf("-demuxer");
         argv[arg++] = g_strdup_printf("lavf");
     }
@@ -1041,8 +1233,16 @@ gpointer launch_player(gpointer data)
 	g_free(filename);
 */
 
-    argv[arg++] = g_strdup_printf("-af-add");
-    argv[arg++] = g_strdup_printf("export=%s:512", idledata->af_export);
+    if (!use_hw_audio) {
+        argv[arg++] = g_strdup_printf("-af-add");
+        argv[arg++] = g_strdup_printf("export=%s:512", idledata->af_export);
+    }
+
+    if (g_strrstr(threaddata->filename, "apple.com")) {
+        argv[arg++] = g_strdup_printf("-user-agent");
+        argv[arg++] = g_strdup_printf("QuickTime/7.6.2");
+    }
+
 
     if (threaddata->playlist)
         argv[arg++] = g_strdup_printf("-playlist");
@@ -1107,13 +1307,13 @@ gpointer launch_player(gpointer data)
         g_io_channel_set_close_on_unref(channel_err, TRUE);
         watch_in_id =
             g_io_add_watch_full(channel_out, G_PRIORITY_LOW, G_IO_IN | G_IO_HUP, thread_reader,
-                                NULL, NULL);
+                                threaddata, NULL);
         watch_err_id =
             g_io_add_watch_full(channel_err, G_PRIORITY_LOW, G_IO_IN | G_IO_ERR | G_IO_HUP,
-                                thread_reader_error, NULL, NULL);
+                                thread_reader_error, threaddata, NULL);
         watch_in_hup_id =
             g_io_add_watch_full(channel_out, G_PRIORITY_LOW, G_IO_ERR | G_IO_HUP, thread_complete,
-                                NULL, NULL);
+                                threaddata, NULL);
 //        watch_in_id = g_io_add_watch(channel_in, G_IO_IN, thread_reader, NULL);
 //        watch_err_id = g_io_add_watch(channel_err, G_IO_IN | G_IO_ERR | G_IO_HUP, thread_reader_error, NULL);
 //        watch_in_hup_id = g_io_add_watch(channel_in, G_IO_ERR | G_IO_HUP, thread_complete, NULL);
@@ -1137,11 +1337,6 @@ gpointer launch_player(gpointer data)
         g_idle_add(set_progress_value, idledata);
         g_idle_add(set_progress_time, idledata);
 
-        if (embed_window != 0 || control_id != 0) {
-            dbus_send_event("MediaComplete", 0);
-            dbus_open_next();
-        }
-
         if (channel_in != NULL) {
             g_io_channel_shutdown(channel_in, FALSE, NULL);
             g_io_channel_unref(channel_in);
@@ -1211,8 +1406,7 @@ gpointer launch_player(gpointer data)
             } else {
                 // printf("end of thread playlist is empty\n");
                 if (loop) {
-                    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
-                    if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+                    if (first_item_in_playlist(&iter)) {
                         gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,
                                            &filename, COUNT_COLUMN, &count, PLAYLIST_COLUMN,
                                            &playlist, -1);
@@ -1228,6 +1422,12 @@ gpointer launch_player(gpointer data)
                     idledata->fullscreen = 0;
                     g_idle_add(set_fullscreen, idledata);
                     g_idle_add(set_stop, idledata);
+
+                    // nothing is on the playlist and we are not looping so ask plugin for next item
+                    if (embed_window != 0 || control_id != 0) {
+                        dbus_send_event("MediaComplete", 0);
+                        dbus_open_next();
+                    }
                 }
 
                 if (quit_on_complete) {
